"""
Python module generated from Java source file java.util.concurrent.locks.Condition

Java source file obtained from artifact jdk version jdk

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from java.util import Date
from java.util.concurrent import TimeUnit
from java.util.concurrent.locks import *
from typing import Any, Callable, Iterable, Tuple


class Condition:
    """
    `Condition` factors out the `Object` monitor
    methods (Object.wait() wait, Object.notify notify
    and Object.notifyAll notifyAll) into distinct objects to
    give the effect of having multiple wait-sets per object, by
    combining them with the use of arbitrary Lock implementations.
    Where a `Lock` replaces the use of `synchronized` methods
    and statements, a `Condition` replaces the use of the Object
    monitor methods.
    
    Conditions (also known as *condition queues* or
    *condition variables*) provide a means for one thread to
    suspend execution (to &quot;wait&quot;) until notified by another
    thread that some state condition may now be True.  Because access
    to this shared state information occurs in different threads, it
    must be protected, so a lock of some form is associated with the
    condition. The key property that waiting for a condition provides
    is that it *atomically* releases the associated lock and
    suspends the current thread, just like `Object.wait`.
    
    A `Condition` instance is intrinsically bound to a lock.
    To obtain a `Condition` instance for a particular Lock
    instance use its Lock.newCondition newCondition() method.
    
    As an example, suppose we have a bounded buffer which supports
    `put` and `take` methods.  If a
    `take` is attempted on an empty buffer, then the thread will block
    until an item becomes available; if a `put` is attempted on a
    full buffer, then the thread will block until a space becomes available.
    We would like to keep waiting `put` threads and `take`
    threads in separate wait-sets so that we can use the optimization of
    only notifying a single thread at a time when items or spaces become
    available in the buffer. This can be achieved using two
    Condition instances.
    ```
    class BoundedBuffer&lt;E&gt; {
      **final Lock lock = new ReentrantLock();**
      final Condition notFull  = **lock.newCondition(); **
      final Condition notEmpty = **lock.newCondition(); **
    
      final Object[] items = new Object[100];
      int putptr, takeptr, count;
    
      public void put(E x) throws InterruptedException {
        **lock.lock();
        try {**
          while (count == items.length)
            **notFull.await();**
          items[putptr] = x;
          if (++putptr == items.length) putptr = 0;
          ++count;
          **notEmpty.signal();**
        **} finally {
          lock.unlock();
        }**
      }
    
      public E take() throws InterruptedException {
        **lock.lock();
        try {**
          while (count == 0)
            **notEmpty.await();**
          E x = (E) items[takeptr];
          if (++takeptr == items.length) takeptr = 0;
          --count;
          **notFull.signal();**
          return x;
        **} finally {
          lock.unlock();
        }**
      }
    }
    ```
    
    (The java.util.concurrent.ArrayBlockingQueue class provides
    this functionality, so there is no reason to implement this
    sample usage class.)
    
    A `Condition` implementation can provide behavior and semantics
    that is
    different from that of the `Object` monitor methods, such as
    guaranteed ordering for notifications, or not requiring a lock to be held
    when performing notifications.
    If an implementation provides such specialized semantics then the
    implementation must document those semantics.
    
    Note that `Condition` instances are just normal objects and can
    themselves be used as the target in a `synchronized` statement,
    and can have their own monitor Object.wait wait and
    Object.notify notify methods invoked.
    Acquiring the monitor lock of a `Condition` instance, or using its
    monitor methods, has no specified relationship with acquiring the
    Lock associated with that `Condition` or the use of its
    .await waiting and .signal signalling methods.
    It is recommended that to avoid confusion you never use `Condition`
    instances in this way, except perhaps within their own implementation.
    
    Except where noted, passing a `null` value for any parameter
    will result in a NullPointerException being thrown.
    
    <h2>Implementation Considerations</h2>
    
    When waiting upon a `Condition`, a &quot;*spurious
    wakeup*&quot; is permitted to occur, in
    general, as a concession to the underlying platform semantics.
    This has little practical impact on most application programs as a
    `Condition` should always be waited upon in a loop, testing
    the state predicate that is being waited for.  An implementation is
    free to remove the possibility of spurious wakeups but it is
    recommended that applications programmers always assume that they can
    occur and so always wait in a loop.
    
    The three forms of condition waiting
    (interruptible, non-interruptible, and timed) may differ in their ease of
    implementation on some platforms and in their performance characteristics.
    In particular, it may be difficult to provide these features and maintain
    specific semantics such as ordering guarantees.
    Further, the ability to interrupt the actual suspension of the thread may
    not always be feasible to implement on all platforms.
    
    Consequently, an implementation is not required to define exactly the
    same guarantees or semantics for all three forms of waiting, nor is it
    required to support interruption of the actual suspension of the thread.
    
    An implementation is required to
    clearly document the semantics and guarantees provided by each of the
    waiting methods, and when an implementation does support interruption of
    thread suspension then it must obey the interruption semantics as defined
    in this interface.
    
    As interruption generally implies cancellation, and checks for
    interruption are often infrequent, an implementation can favor responding
    to an interrupt over normal method return. This is True even if it can be
    shown that the interrupt occurred after another action that may have
    unblocked the thread. An implementation should document this behavior.

    Author(s)
    - Doug Lea

    Since
    - 1.5
    """

    def await(self) -> None:
        """
        Causes the current thread to wait until it is signalled or
        Thread.interrupt interrupted.
        
        The lock associated with this `Condition` is atomically
        released and the current thread becomes disabled for thread scheduling
        purposes and lies dormant until *one* of four things happens:
        
        - Some other thread invokes the .signal method for this
        `Condition` and the current thread happens to be chosen as the
        thread to be awakened; or
        - Some other thread invokes the .signalAll method for this
        `Condition`; or
        - Some other thread Thread.interrupt interrupts the
        current thread, and interruption of thread suspension is supported; or
        - A &quot;*spurious wakeup*&quot; occurs.
        
        
        In all cases, before this method can return the current thread must
        re-acquire the lock associated with this condition. When the
        thread returns it is *guaranteed* to hold this lock.
        
        If the current thread:
        
        - has its interrupted status set on entry to this method; or
        - is Thread.interrupt interrupted while waiting
        and interruption of thread suspension is supported,
        
        then InterruptedException is thrown and the current thread's
        interrupted status is cleared. It is not specified, in the first
        case, whether or not the test for interruption occurs before the lock
        is released.
        
        **Implementation Considerations**
        
        The current thread is assumed to hold the lock associated with this
        `Condition` when this method is called.
        It is up to the implementation to determine if this is
        the case and if not, how to respond. Typically, an exception will be
        thrown (such as IllegalMonitorStateException) and the
        implementation must document that fact.
        
        An implementation can favor responding to an interrupt over normal
        method return in response to a signal. In that case the implementation
        must ensure that the signal is redirected to another waiting thread, if
        there is one.

        Raises
        - InterruptedException: if the current thread is interrupted
                (and interruption of thread suspension is supported)
        """
        ...


    def awaitUninterruptibly(self) -> None:
        """
        Causes the current thread to wait until it is signalled.
        
        The lock associated with this condition is atomically
        released and the current thread becomes disabled for thread scheduling
        purposes and lies dormant until *one* of three things happens:
        
        - Some other thread invokes the .signal method for this
        `Condition` and the current thread happens to be chosen as the
        thread to be awakened; or
        - Some other thread invokes the .signalAll method for this
        `Condition`; or
        - A &quot;*spurious wakeup*&quot; occurs.
        
        
        In all cases, before this method can return the current thread must
        re-acquire the lock associated with this condition. When the
        thread returns it is *guaranteed* to hold this lock.
        
        If the current thread's interrupted status is set when it enters
        this method, or it is Thread.interrupt interrupted
        while waiting, it will continue to wait until signalled. When it finally
        returns from this method its interrupted status will still
        be set.
        
        **Implementation Considerations**
        
        The current thread is assumed to hold the lock associated with this
        `Condition` when this method is called.
        It is up to the implementation to determine if this is
        the case and if not, how to respond. Typically, an exception will be
        thrown (such as IllegalMonitorStateException) and the
        implementation must document that fact.
        """
        ...


    def awaitNanos(self, nanosTimeout: int) -> int:
        """
        Causes the current thread to wait until it is signalled or interrupted,
        or the specified waiting time elapses.
        
        The lock associated with this condition is atomically
        released and the current thread becomes disabled for thread scheduling
        purposes and lies dormant until *one* of five things happens:
        
        - Some other thread invokes the .signal method for this
        `Condition` and the current thread happens to be chosen as the
        thread to be awakened; or
        - Some other thread invokes the .signalAll method for this
        `Condition`; or
        - Some other thread Thread.interrupt interrupts the
        current thread, and interruption of thread suspension is supported; or
        - The specified waiting time elapses; or
        - A &quot;*spurious wakeup*&quot; occurs.
        
        
        In all cases, before this method can return the current thread must
        re-acquire the lock associated with this condition. When the
        thread returns it is *guaranteed* to hold this lock.
        
        If the current thread:
        
        - has its interrupted status set on entry to this method; or
        - is Thread.interrupt interrupted while waiting
        and interruption of thread suspension is supported,
        
        then InterruptedException is thrown and the current thread's
        interrupted status is cleared. It is not specified, in the first
        case, whether or not the test for interruption occurs before the lock
        is released.
        
        The method returns an estimate of the number of nanoseconds
        remaining to wait given the supplied `nanosTimeout`
        value upon return, or a value less than or equal to zero if it
        timed out. This value can be used to determine whether and how
        long to re-wait in cases where the wait returns but an awaited
        condition still does not hold. Typical uses of this method take
        the following form:
        
        ``` `boolean aMethod(long timeout, TimeUnit unit)
            throws InterruptedException {
          long nanosRemaining = unit.toNanos(timeout);
          lock.lock();
          try {
            while (!conditionBeingWaitedFor()) {
              if (nanosRemaining <= 0L)
                return False;
              nanosRemaining = theCondition.awaitNanos(nanosRemaining);`
            // ...
            return True;
          } finally {
            lock.unlock();
          }
        }}```
        
        Design note: This method requires a nanosecond argument so
        as to avoid truncation errors in reporting remaining times.
        Such precision loss would make it difficult for programmers to
        ensure that total waiting times are not systematically shorter
        than specified when re-waits occur.
        
        **Implementation Considerations**
        
        The current thread is assumed to hold the lock associated with this
        `Condition` when this method is called.
        It is up to the implementation to determine if this is
        the case and if not, how to respond. Typically, an exception will be
        thrown (such as IllegalMonitorStateException) and the
        implementation must document that fact.
        
        An implementation can favor responding to an interrupt over normal
        method return in response to a signal, or over indicating the elapse
        of the specified waiting time. In either case the implementation
        must ensure that the signal is redirected to another waiting thread, if
        there is one.

        Arguments
        - nanosTimeout: the maximum time to wait, in nanoseconds

        Returns
        - an estimate of the `nanosTimeout` value minus
                the time spent waiting upon return from this method.
                A positive value may be used as the argument to a
                subsequent call to this method to finish waiting out
                the desired time.  A value less than or equal to zero
                indicates that no time remains.

        Raises
        - InterruptedException: if the current thread is interrupted
                (and interruption of thread suspension is supported)
        """
        ...


    def await(self, time: int, unit: "TimeUnit") -> bool:
        """
        Causes the current thread to wait until it is signalled or interrupted,
        or the specified waiting time elapses. This method is behaviorally
        equivalent to:
        ``` `awaitNanos(unit.toNanos(time)) > 0````

        Arguments
        - time: the maximum time to wait
        - unit: the time unit of the `time` argument

        Returns
        - `False` if the waiting time detectably elapsed
                before return from the method, else `True`

        Raises
        - InterruptedException: if the current thread is interrupted
                (and interruption of thread suspension is supported)
        """
        ...


    def awaitUntil(self, deadline: "Date") -> bool:
        """
        Causes the current thread to wait until it is signalled or interrupted,
        or the specified deadline elapses.
        
        The lock associated with this condition is atomically
        released and the current thread becomes disabled for thread scheduling
        purposes and lies dormant until *one* of five things happens:
        
        - Some other thread invokes the .signal method for this
        `Condition` and the current thread happens to be chosen as the
        thread to be awakened; or
        - Some other thread invokes the .signalAll method for this
        `Condition`; or
        - Some other thread Thread.interrupt interrupts the
        current thread, and interruption of thread suspension is supported; or
        - The specified deadline elapses; or
        - A &quot;*spurious wakeup*&quot; occurs.
        
        
        In all cases, before this method can return the current thread must
        re-acquire the lock associated with this condition. When the
        thread returns it is *guaranteed* to hold this lock.
        
        If the current thread:
        
        - has its interrupted status set on entry to this method; or
        - is Thread.interrupt interrupted while waiting
        and interruption of thread suspension is supported,
        
        then InterruptedException is thrown and the current thread's
        interrupted status is cleared. It is not specified, in the first
        case, whether or not the test for interruption occurs before the lock
        is released.
        
        The return value indicates whether the deadline has elapsed,
        which can be used as follows:
        ``` `boolean aMethod(Date deadline)
            throws InterruptedException {
          boolean stillWaiting = True;
          lock.lock();
          try {
            while (!conditionBeingWaitedFor()) {
              if (!stillWaiting)
                return False;
              stillWaiting = theCondition.awaitUntil(deadline);`
            // ...
            return True;
          } finally {
            lock.unlock();
          }
        }}```
        
        **Implementation Considerations**
        
        The current thread is assumed to hold the lock associated with this
        `Condition` when this method is called.
        It is up to the implementation to determine if this is
        the case and if not, how to respond. Typically, an exception will be
        thrown (such as IllegalMonitorStateException) and the
        implementation must document that fact.
        
        An implementation can favor responding to an interrupt over normal
        method return in response to a signal, or over indicating the passing
        of the specified deadline. In either case the implementation
        must ensure that the signal is redirected to another waiting thread, if
        there is one.

        Arguments
        - deadline: the absolute time to wait until

        Returns
        - `False` if the deadline has elapsed upon return, else
                `True`

        Raises
        - InterruptedException: if the current thread is interrupted
                (and interruption of thread suspension is supported)
        """
        ...


    def signal(self) -> None:
        """
        Wakes up one waiting thread.
        
        If any threads are waiting on this condition then one
        is selected for waking up. That thread must then re-acquire the
        lock before returning from `await`.
        
        **Implementation Considerations**
        
        An implementation may (and typically does) require that the
        current thread hold the lock associated with this `Condition` when this method is called. Implementations must
        document this precondition and any actions taken if the lock is
        not held. Typically, an exception such as IllegalMonitorStateException will be thrown.
        """
        ...


    def signalAll(self) -> None:
        """
        Wakes up all waiting threads.
        
        If any threads are waiting on this condition then they are
        all woken up. Each thread must re-acquire the lock before it can
        return from `await`.
        
        **Implementation Considerations**
        
        An implementation may (and typically does) require that the
        current thread hold the lock associated with this `Condition` when this method is called. Implementations must
        document this precondition and any actions taken if the lock is
        not held. Typically, an exception such as IllegalMonitorStateException will be thrown.
        """
        ...



"""
Python module generated from Java source file org.bukkit.inventory.ShapedRecipe

Java source file obtained from artifact spigot-api version 1.21.4-R0.1-20250303.102353-42

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from com.google.common.base import Preconditions
from java.util import Collections
from org.bukkit import Material
from org.bukkit import NamespacedKey
from org.bukkit.inventory import *
from org.bukkit.material import MaterialData
from typing import Any, Callable, Iterable, Tuple


class ShapedRecipe(CraftingRecipe):
    """
    Represents a shaped (ie normal) crafting recipe.
    """

    def __init__(self, result: "ItemStack"):
        """
        Create a shaped recipe to craft the specified ItemStack. The
        constructor merely determines the result and type; to set the actual
        recipe, you'll need to call the appropriate methods.

        Arguments
        - result: The item you want the recipe to create.

        See
        - ShapedRecipe.setIngredient(char, RecipeChoice)

        Deprecated
        - Recipes must have keys. Use .ShapedRecipe(NamespacedKey, ItemStack)
        instead.
        """
        ...


    def __init__(self, key: "NamespacedKey", result: "ItemStack"):
        """
        Create a shaped recipe to craft the specified ItemStack. The
        constructor merely determines the result and type; to set the actual
        recipe, you'll need to call the appropriate methods.

        Arguments
        - key: the unique recipe key
        - result: The item you want the recipe to create.

        Raises
        - IllegalArgumentException: if the `result` is an empty item (AIR)

        See
        - ShapedRecipe.setIngredient(char, RecipeChoice)
        """
        ...


    def shape(self, *shape: Tuple[str, ...]) -> "ShapedRecipe":
        """
        Set the shape of this recipe to the specified rows. Each character
        represents a different ingredient; excluding space characters, which
        must be empty, exactly what each character represents is set separately.
        The first row supplied corresponds with the upper most part of the recipe
        on the workbench e.g. if all three rows are supplies the first string
        represents the top row on the workbench.

        Arguments
        - shape: The rows of the recipe (up to 3 rows).

        Returns
        - The changed recipe, so you can chain calls.
        """
        ...


    def setIngredient(self, key: str, ingredient: "MaterialData") -> "ShapedRecipe":
        """
        Sets the material that a character in the recipe shape refers to.
        
        Note that before an ingredient can be set, the recipe's shape must be defined
        with .shape(String...).

        Arguments
        - key: The character that represents the ingredient in the shape.
        - ingredient: The ingredient.

        Returns
        - The changed recipe, so you can chain calls.

        Raises
        - IllegalArgumentException: if the `key` is a space character
        - IllegalArgumentException: if the `key` does not appear in the shape.
        """
        ...


    def setIngredient(self, key: str, ingredient: "Material") -> "ShapedRecipe":
        """
        Sets the material that a character in the recipe shape refers to.
        
        Note that before an ingredient can be set, the recipe's shape must be defined
        with .shape(String...).

        Arguments
        - key: The character that represents the ingredient in the shape.
        - ingredient: The ingredient.

        Returns
        - The changed recipe, so you can chain calls.

        Raises
        - IllegalArgumentException: if the `key` is a space character
        - IllegalArgumentException: if the `key` does not appear in the shape.
        """
        ...


    def setIngredient(self, key: str, ingredient: "Material", raw: int) -> "ShapedRecipe":
        """
        Sets the material that a character in the recipe shape refers to.
        
        Note that before an ingredient can be set, the recipe's shape must be defined
        with .shape(String...).

        Arguments
        - key: The character that represents the ingredient in the shape.
        - ingredient: The ingredient.
        - raw: The raw material data as an integer.

        Returns
        - The changed recipe, so you can chain calls.

        Raises
        - IllegalArgumentException: if the `key` is a space character
        - IllegalArgumentException: if the `key` does not appear in the shape.

        Deprecated
        - Magic value
        """
        ...


    def setIngredient(self, key: str, ingredient: "RecipeChoice") -> "ShapedRecipe":
        """
        Sets the RecipeChoice that a character in the recipe shape refers to.
        
        Note that before an ingredient can be set, the recipe's shape must be defined
        with .shape(String...).

        Arguments
        - key: The character that represents the ingredient in the shape.
        - ingredient: The ingredient.

        Returns
        - The changed recipe, so you can chain calls.

        Raises
        - IllegalArgumentException: if the `key` is a space character
        - IllegalArgumentException: if the `key` does not appear in the shape.
        """
        ...


    def getIngredientMap(self) -> dict["Character", "ItemStack"]:
        """
        Get a copy of the ingredients map.

        Returns
        - The mapping of character to ingredients.
        """
        ...


    def getChoiceMap(self) -> dict["Character", "RecipeChoice"]:
        """
        Get a copy of the choice map.

        Returns
        - The mapping of character to ingredients.
        """
        ...


    def getShape(self) -> list[str]:
        """
        Get the shape.

        Returns
        - The recipe's shape.

        Raises
        - NullPointerException: when not set yet
        """
        ...



"""
Python module generated from Java source file org.bukkit.potion.PotionData

Java source file obtained from artifact spigot-api version 1.21.4-R0.1-20250303.102353-42

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from com.google.common.base import Preconditions
from org.bukkit.potion import *
from typing import Any, Callable, Iterable, Tuple


class PotionData:
    """
    Deprecated
    - Upgraded / extended potions are now their own PotionType use them instead.
    """

    def __init__(self, type: "PotionType", extended: bool, upgraded: bool):
        """
        Instantiates a final PotionData object to contain information about a
        Potion

        Arguments
        - type: the type of the Potion
        - extended: whether the potion is extended PotionType#isExtendable()
        must be True
        - upgraded: whether the potion is upgraded PotionType#isUpgradable()
        must be True
        """
        ...


    def __init__(self, type: "PotionType"):
        ...


    def getType(self) -> "PotionType":
        """
        Gets the type of the potion, Type matches up with each kind of craftable
        potion

        Returns
        - the potion type
        """
        ...


    def isUpgraded(self) -> bool:
        """
        Checks if the potion is in an upgraded state. This refers to whether or
        not the potion is Tier 2, such as Potion of Fire Resistance II.

        Returns
        - True if the potion is upgraded;
        """
        ...


    def isExtended(self) -> bool:
        """
        Checks if the potion is in an extended state. This refers to the extended
        duration potions

        Returns
        - True if the potion is extended
        """
        ...


    def hashCode(self) -> int:
        ...


    def equals(self, obj: "Object") -> bool:
        ...



"""
Python module generated from Java source file com.google.common.collect.ListMultimap

Java source file obtained from artifact guava version 33.3.1-jre

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from com.google.common.annotations import GwtCompatible
from com.google.common.collect import *
from com.google.errorprone.annotations import CanIgnoreReturnValue
from javax.annotation import CheckForNull
from org.checkerframework.checker.nullness.qual import Nullable
from typing import Any, Callable, Iterable, Tuple


class ListMultimap(Multimap):
    """
    A `Multimap` that can hold duplicate key-value pairs and that maintains the insertion
    ordering of values for a given key. See the Multimap documentation for information common
    to all multimaps.
    
    The .get, .removeAll, and .replaceValues methods each return a List of values. Though the method signature doesn't say so explicitly, the map returned by
    .asMap has `List` values.
    
    See the Guava User Guide article on <a href=
    "https://github.com/google/guava/wiki/NewCollectionTypesExplained#multimap">`Multimap`</a>.

    Author(s)
    - Jared Levy

    Since
    - 2.0
    """

    def get(self, key: "K") -> list["V"]:
        """
        
        
        Because the values for a given key may have duplicates and follow the insertion ordering,
        this method returns a List, instead of the java.util.Collection specified in
        the Multimap interface.
        """
        ...


    def removeAll(self, key: "Object") -> list["V"]:
        """
        
        
        Because the values for a given key may have duplicates and follow the insertion ordering,
        this method returns a List, instead of the java.util.Collection specified in
        the Multimap interface.
        """
        ...


    def replaceValues(self, key: "K", values: Iterable["V"]) -> list["V"]:
        """
        
        
        Because the values for a given key may have duplicates and follow the insertion ordering,
        this method returns a List, instead of the java.util.Collection specified in
        the Multimap interface.
        """
        ...


    def asMap(self) -> dict["K", Iterable["V"]]:
        """
        
        
        **Note:** The returned map's values are guaranteed to be of type List. To obtain
        this map with the more specific generic type `Map<K, List<V>>`, call Multimaps.asMap(ListMultimap) instead.
        """
        ...


    def equals(self, obj: "Object") -> bool:
        """
        Compares the specified object to this multimap for equality.
        
        Two `ListMultimap` instances are equal if, for each key, they contain the same values
        in the same order. If the value orderings disagree, the multimaps will not be considered equal.
        
        An empty `ListMultimap` is equal to any other empty `Multimap`, including an
        empty `SetMultimap`.
        """
        ...



"""
Python module generated from Java source file org.bukkit.event.inventory.InventoryCloseEvent

Java source file obtained from artifact spigot-api version 1.21.4-R0.1-20250303.102353-42

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from org.bukkit.entity import HumanEntity
from org.bukkit.event import HandlerList
from org.bukkit.event.inventory import *
from org.bukkit.inventory import InventoryView
from typing import Any, Callable, Iterable, Tuple


class InventoryCloseEvent(InventoryEvent):
    """
    Represents a player related inventory event
    """

    def __init__(self, transaction: "InventoryView"):
        ...


    def getPlayer(self) -> "HumanEntity":
        """
        Returns the player involved in this event

        Returns
        - Player who is involved in this event
        """
        ...


    def getHandlers(self) -> "HandlerList":
        ...


    @staticmethod
    def getHandlerList() -> "HandlerList":
        ...



"""
Python module generated from Java source file org.joml.Matrix3x2d

Java source file obtained from artifact joml version 1.10.8

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from java.io import Externalizable
from java.io import IOException
from java.io import ObjectInput
from java.io import ObjectOutput
from java.text import DecimalFormat
from java.text import NumberFormat
from org.joml import *
from typing import Any, Callable, Iterable, Tuple


class Matrix3x2d(Matrix3x2dc, Cloneable, Externalizable):
    """
    Contains the definition of a 3x2 matrix of doubles, and associated functions to transform
    it. The matrix is column-major to match OpenGL's interpretation, and it looks like this:
    
         m00  m10  m20
         m01  m11  m21

    Author(s)
    - Kai Burjack
    """

    def __init__(self):
        """
        Create a new Matrix3x2d and set it to .identity() identity.
        """
        ...


    def __init__(self, mat: "Matrix2dc"):
        """
        Create a new Matrix3x2d by setting its left 2x2 submatrix to the values of the given Matrix2dc
        and the rest to identity.

        Arguments
        - mat: the Matrix2dc
        """
        ...


    def __init__(self, mat: "Matrix2fc"):
        """
        Create a new Matrix3x2d by setting its left 2x2 submatrix to the values of the given Matrix2fc
        and the rest to identity.

        Arguments
        - mat: the Matrix2fc
        """
        ...


    def __init__(self, mat: "Matrix3x2dc"):
        """
        Create a new Matrix3x2d and make it a copy of the given matrix.

        Arguments
        - mat: the Matrix3x2dc to copy the values from
        """
        ...


    def __init__(self, m00: float, m01: float, m10: float, m11: float, m20: float, m21: float):
        """
        Create a new 3x2 matrix using the supplied double values. The order of the parameter is column-major, 
        so the first two parameters specify the two elements of the first column.

        Arguments
        - m00: the value of m00
        - m01: the value of m01
        - m10: the value of m10
        - m11: the value of m11
        - m20: the value of m20
        - m21: the value of m21
        """
        ...


    def __init__(self, buffer: "DoubleBuffer"):
        """
        Create a new Matrix3x2d by reading its 6 double components from the given DoubleBuffer
        at the buffer's current position.
        
        That DoubleBuffer is expected to hold the values in column-major order.
        
        The buffer's position will not be changed by this method.

        Arguments
        - buffer: the DoubleBuffer to read the matrix values from
        """
        ...


    def m00(self) -> float:
        ...


    def m01(self) -> float:
        ...


    def m10(self) -> float:
        ...


    def m11(self) -> float:
        ...


    def m20(self) -> float:
        ...


    def m21(self) -> float:
        ...


    def set(self, m: "Matrix3x2dc") -> "Matrix3x2d":
        """
        Set the elements of this matrix to the ones in `m`.

        Arguments
        - m: the matrix to copy the elements from

        Returns
        - this
        """
        ...


    def set(self, m: "Matrix2dc") -> "Matrix3x2d":
        """
        Set the left 2x2 submatrix of this Matrix3x2d to the given Matrix2dc and don't change the other elements.

        Arguments
        - m: the 2x2 matrix

        Returns
        - this
        """
        ...


    def set(self, m: "Matrix2fc") -> "Matrix3x2d":
        """
        Set the left 2x2 submatrix of this Matrix3x2d to the given Matrix2fc and don't change the other elements.

        Arguments
        - m: the 2x2 matrix

        Returns
        - this
        """
        ...


    def mul(self, right: "Matrix3x2dc") -> "Matrix3x2d":
        """
        Multiply this matrix by the supplied `right` matrix by assuming a third row in
        both matrices of `(0, 0, 1)`.
        
        If `M` is `this` matrix and `R` the `right` matrix,
        then the new matrix will be `M * R`. So when transforming a
        vector `v` with the new matrix by using `M * R * v`, the
        transformation of the right matrix will be applied first!

        Arguments
        - right: the right operand of the matrix multiplication

        Returns
        - this
        """
        ...


    def mul(self, right: "Matrix3x2dc", dest: "Matrix3x2d") -> "Matrix3x2d":
        """
        Multiply this matrix by the supplied `right` matrix by assuming a third row in
        both matrices of `(0, 0, 1)` and store the result in `dest`.
        
        If `M` is `this` matrix and `R` the `right` matrix,
        then the new matrix will be `M * R`. So when transforming a
        vector `v` with the new matrix by using `M * R * v`, the
        transformation of the right matrix will be applied first!

        Arguments
        - right: the right operand of the matrix multiplication
        - dest: will hold the result

        Returns
        - dest
        """
        ...


    def mulLocal(self, left: "Matrix3x2dc") -> "Matrix3x2d":
        """
        Pre-multiply this matrix by the supplied `left` matrix and store the result in `this`.
        
        If `M` is `this` matrix and `L` the `left` matrix,
        then the new matrix will be `L * M`. So when transforming a
        vector `v` with the new matrix by using `L * M * v`, the
        transformation of `this` matrix will be applied first!

        Arguments
        - left: the left operand of the matrix multiplication

        Returns
        - this
        """
        ...


    def mulLocal(self, left: "Matrix3x2dc", dest: "Matrix3x2d") -> "Matrix3x2d":
        ...


    def set(self, m00: float, m01: float, m10: float, m11: float, m20: float, m21: float) -> "Matrix3x2d":
        """
        Set the values within this matrix to the supplied double values. The result looks like this:
        
        m00, m10, m20
        m01, m11, m21

        Arguments
        - m00: the new value of m00
        - m01: the new value of m01
        - m10: the new value of m10
        - m11: the new value of m11
        - m20: the new value of m20
        - m21: the new value of m21

        Returns
        - this
        """
        ...


    def set(self, m: list[float]) -> "Matrix3x2d":
        """
        Set the values in this matrix based on the supplied float array in column-major order. The result looks like this:
        
        0, 2, 4
        1, 3, 5
        
        This method only uses the first 6 values, all others are ignored.

        Arguments
        - m: the array to read the matrix values from

        Returns
        - this
        """
        ...


    def set(self, m: list[float], off: int) -> "Matrix3x2d":
        """
        Set the values in this matrix based on the supplied float array in column-major order. The result looks like this:
        
        0, 2, 4
        1, 3, 5
        
        This method only uses the 6 values starting at the given offset.

        Arguments
        - m: the array to read the matrix values from
        - off: the offset into the array

        Returns
        - this
        """
        ...


    def set(self, m: list[float]) -> "Matrix3x2d":
        """
        Set the values in this matrix based on the supplied double array in column-major order. The result looks like this:
        
        0, 2, 4
        1, 3, 5
        
        This method only uses the first 6 values, all others are ignored.

        Arguments
        - m: the array to read the matrix values from

        Returns
        - this
        """
        ...


    def set(self, m: list[float], off: int) -> "Matrix3x2d":
        """
        Set the values in this matrix based on the supplied double array in column-major order. The result looks like this:
        
        0, 2, 4
        1, 3, 5
        
        This method only uses the 6 values starting at the given offset.

        Arguments
        - m: the array to read the matrix values from
        - off: the offset into the array

        Returns
        - this
        """
        ...


    def determinant(self) -> float:
        """
        Return the determinant of this matrix.

        Returns
        - the determinant
        """
        ...


    def invert(self) -> "Matrix3x2d":
        """
        Invert this matrix by assuming a third row in this matrix of `(0, 0, 1)`.

        Returns
        - this
        """
        ...


    def invert(self, dest: "Matrix3x2d") -> "Matrix3x2d":
        """
        Invert the `this` matrix by assuming a third row in this matrix of `(0, 0, 1)`
        and store the result in `dest`.

        Arguments
        - dest: will hold the result

        Returns
        - dest
        """
        ...


    def translation(self, x: float, y: float) -> "Matrix3x2d":
        """
        Set this matrix to be a simple translation matrix in a two-dimensional coordinate system.
        
        The resulting matrix can be multiplied against another transformation
        matrix to obtain an additional translation.
        
        In order to apply a translation via to an already existing transformation
        matrix, use .translate(double, double) translate() instead.

        Arguments
        - x: the units to translate in x
        - y: the units to translate in y

        Returns
        - this

        See
        - .translate(double, double)
        """
        ...


    def translation(self, offset: "Vector2dc") -> "Matrix3x2d":
        """
        Set this matrix to be a simple translation matrix in a two-dimensional coordinate system.
        
        The resulting matrix can be multiplied against another transformation
        matrix to obtain an additional translation.
        
        In order to apply a translation via to an already existing transformation
        matrix, use .translate(Vector2dc) translate() instead.

        Arguments
        - offset: the translation

        Returns
        - this

        See
        - .translate(Vector2dc)
        """
        ...


    def setTranslation(self, x: float, y: float) -> "Matrix3x2d":
        """
        Set only the translation components of this matrix `(m20, m21)` to the given values `(x, y)`.
        
        To build a translation matrix instead, use .translation(double, double).
        To apply a translation to another matrix, use .translate(double, double).

        Arguments
        - x: the offset to translate in x
        - y: the offset to translate in y

        Returns
        - this

        See
        - .translate(double, double)
        """
        ...


    def setTranslation(self, offset: "Vector2dc") -> "Matrix3x2d":
        """
        Set only the translation components of this matrix `(m20, m21)` to the given values `(offset.x, offset.y)`.
        
        To build a translation matrix instead, use .translation(Vector2dc).
        To apply a translation to another matrix, use .translate(Vector2dc).

        Arguments
        - offset: the new translation to set

        Returns
        - this

        See
        - .translate(Vector2dc)
        """
        ...


    def translate(self, x: float, y: float, dest: "Matrix3x2d") -> "Matrix3x2d":
        """
        Apply a translation to this matrix by translating by the given number of units in x and y and store the result
        in `dest`.
        
        If `M` is `this` matrix and `T` the translation
        matrix, then the new matrix will be `M * T`. So when
        transforming a vector `v` with the new matrix by using
        `M * T * v`, the translation will be applied first!
        
        In order to set the matrix to a translation transformation without post-multiplying
        it, use .translation(double, double).

        Arguments
        - x: the offset to translate in x
        - y: the offset to translate in y
        - dest: will hold the result

        Returns
        - dest

        See
        - .translation(double, double)
        """
        ...


    def translate(self, x: float, y: float) -> "Matrix3x2d":
        """
        Apply a translation to this matrix by translating by the given number of units in x and y.
        
        If `M` is `this` matrix and `T` the translation
        matrix, then the new matrix will be `M * T`. So when
        transforming a vector `v` with the new matrix by using
        `M * T * v`, the translation will be applied first!
        
        In order to set the matrix to a translation transformation without post-multiplying
        it, use .translation(double, double).

        Arguments
        - x: the offset to translate in x
        - y: the offset to translate in y

        Returns
        - this

        See
        - .translation(double, double)
        """
        ...


    def translate(self, offset: "Vector2dc", dest: "Matrix3x2d") -> "Matrix3x2d":
        """
        Apply a translation to this matrix by translating by the given number of units in x and y, and
        store the result in `dest`.
        
        If `M` is `this` matrix and `T` the translation
        matrix, then the new matrix will be `M * T`. So when
        transforming a vector `v` with the new matrix by using
        `M * T * v`, the translation will be applied first!
        
        In order to set the matrix to a translation transformation without post-multiplying
        it, use .translation(Vector2dc).

        Arguments
        - offset: the offset to translate
        - dest: will hold the result

        Returns
        - dest

        See
        - .translation(Vector2dc)
        """
        ...


    def translate(self, offset: "Vector2dc") -> "Matrix3x2d":
        """
        Apply a translation to this matrix by translating by the given number of units in x and y.
        
        If `M` is `this` matrix and `T` the translation
        matrix, then the new matrix will be `M * T`. So when
        transforming a vector `v` with the new matrix by using
        `M * T * v`, the translation will be applied first!
        
        In order to set the matrix to a translation transformation without post-multiplying
        it, use .translation(Vector2dc).

        Arguments
        - offset: the offset to translate

        Returns
        - this

        See
        - .translation(Vector2dc)
        """
        ...


    def translateLocal(self, offset: "Vector2dc") -> "Matrix3x2d":
        """
        Pre-multiply a translation to this matrix by translating by the given number of
        units in x and y.
        
        If `M` is `this` matrix and `T` the translation
        matrix, then the new matrix will be `T * M`. So when
        transforming a vector `v` with the new matrix by using
        `T * M * v`, the translation will be applied last!
        
        In order to set the matrix to a translation transformation without pre-multiplying
        it, use .translation(Vector2dc).

        Arguments
        - offset: the number of units in x and y by which to translate

        Returns
        - this

        See
        - .translation(Vector2dc)
        """
        ...


    def translateLocal(self, offset: "Vector2dc", dest: "Matrix3x2d") -> "Matrix3x2d":
        """
        Pre-multiply a translation to this matrix by translating by the given number of
        units in x and y and store the result in `dest`.
        
        If `M` is `this` matrix and `T` the translation
        matrix, then the new matrix will be `T * M`. So when
        transforming a vector `v` with the new matrix by using
        `T * M * v`, the translation will be applied last!
        
        In order to set the matrix to a translation transformation without pre-multiplying
        it, use .translation(Vector2dc).

        Arguments
        - offset: the number of units in x and y by which to translate
        - dest: will hold the result

        Returns
        - dest

        See
        - .translation(Vector2dc)
        """
        ...


    def translateLocal(self, x: float, y: float, dest: "Matrix3x2d") -> "Matrix3x2d":
        """
        Pre-multiply a translation to this matrix by translating by the given number of
        units in x and y and store the result in `dest`.
        
        If `M` is `this` matrix and `T` the translation
        matrix, then the new matrix will be `T * M`. So when
        transforming a vector `v` with the new matrix by using
        `T * M * v`, the translation will be applied last!
        
        In order to set the matrix to a translation transformation without pre-multiplying
        it, use .translation(double, double).

        Arguments
        - x: the offset to translate in x
        - y: the offset to translate in y
        - dest: will hold the result

        Returns
        - dest

        See
        - .translation(double, double)
        """
        ...


    def translateLocal(self, x: float, y: float) -> "Matrix3x2d":
        """
        Pre-multiply a translation to this matrix by translating by the given number of
        units in x and y.
        
        If `M` is `this` matrix and `T` the translation
        matrix, then the new matrix will be `T * M`. So when
        transforming a vector `v` with the new matrix by using
        `T * M * v`, the translation will be applied last!
        
        In order to set the matrix to a translation transformation without pre-multiplying
        it, use .translation(double, double).

        Arguments
        - x: the offset to translate in x
        - y: the offset to translate in y

        Returns
        - this

        See
        - .translation(double, double)
        """
        ...


    def toString(self) -> str:
        """
        Return a string representation of this matrix.
        
        This method creates a new DecimalFormat on every invocation with the format string "`0.000E0;-`".

        Returns
        - the string representation
        """
        ...


    def toString(self, formatter: "NumberFormat") -> str:
        """
        Return a string representation of this matrix by formatting the matrix elements with the given NumberFormat.

        Arguments
        - formatter: the NumberFormat used to format the matrix values with

        Returns
        - the string representation
        """
        ...


    def get(self, dest: "Matrix3x2d") -> "Matrix3x2d":
        """
        Get the current values of `this` matrix and store them into
        `dest`.
        
        This is the reverse method of .set(Matrix3x2dc) and allows to obtain
        intermediate calculation results when chaining multiple transformations.

        Arguments
        - dest: the destination matrix

        Returns
        - dest

        See
        - .set(Matrix3x2dc)
        """
        ...


    def get(self, buffer: "DoubleBuffer") -> "DoubleBuffer":
        """
        Store this matrix in column-major order into the supplied DoubleBuffer at the current
        buffer DoubleBuffer.position() position.
        
        This method will not increment the position of the given DoubleBuffer.
        
        In order to specify the offset into the DoubleBuffer at which
        the matrix is stored, use .get(int, DoubleBuffer), taking
        the absolute position as parameter.

        Arguments
        - buffer: will receive the values of this matrix in column-major order at its current position

        Returns
        - the passed in buffer

        See
        - .get(int, DoubleBuffer)
        """
        ...


    def get(self, index: int, buffer: "DoubleBuffer") -> "DoubleBuffer":
        """
        Store this matrix in column-major order into the supplied DoubleBuffer starting at the specified
        absolute buffer position/index.
        
        This method will not increment the position of the given DoubleBuffer.

        Arguments
        - index: the absolute position into the DoubleBuffer
        - buffer: will receive the values of this matrix in column-major order

        Returns
        - the passed in buffer
        """
        ...


    def get(self, buffer: "ByteBuffer") -> "ByteBuffer":
        """
        Store this matrix in column-major order into the supplied ByteBuffer at the current
        buffer ByteBuffer.position() position.
        
        This method will not increment the position of the given ByteBuffer.
        
        In order to specify the offset into the ByteBuffer at which
        the matrix is stored, use .get(int, ByteBuffer), taking
        the absolute position as parameter.

        Arguments
        - buffer: will receive the values of this matrix in column-major order at its current position

        Returns
        - the passed in buffer

        See
        - .get(int, ByteBuffer)
        """
        ...


    def get(self, index: int, buffer: "ByteBuffer") -> "ByteBuffer":
        """
        Store this matrix in column-major order into the supplied ByteBuffer starting at the specified
        absolute buffer position/index.
        
        This method will not increment the position of the given ByteBuffer.

        Arguments
        - index: the absolute position into the ByteBuffer
        - buffer: will receive the values of this matrix in column-major order

        Returns
        - the passed in buffer
        """
        ...


    def get3x3(self, buffer: "DoubleBuffer") -> "DoubleBuffer":
        """
        Store this matrix as an equivalent 4x4 matrix in column-major order into the supplied DoubleBuffer at the current
        buffer DoubleBuffer.position() position.
        
        This method will not increment the position of the given DoubleBuffer.
        
        In order to specify the offset into the DoubleBuffer at which
        the matrix is stored, use .get3x3(int, DoubleBuffer), taking
        the absolute position as parameter.

        Arguments
        - buffer: will receive the values of this matrix in column-major order at its current position

        Returns
        - the passed in buffer

        See
        - .get3x3(int, DoubleBuffer)
        """
        ...


    def get3x3(self, index: int, buffer: "DoubleBuffer") -> "DoubleBuffer":
        """
        Store this matrix as an equivalent 3x3 matrix in column-major order into the supplied DoubleBuffer starting at the specified
        absolute buffer position/index.
        
        This method will not increment the position of the given DoubleBuffer.

        Arguments
        - index: the absolute position into the DoubleBuffer
        - buffer: will receive the values of this matrix in column-major order

        Returns
        - the passed in buffer
        """
        ...


    def get3x3(self, buffer: "ByteBuffer") -> "ByteBuffer":
        """
        Store this matrix as an equivalent 3x3 matrix in column-major order into the supplied ByteBuffer at the current
        buffer ByteBuffer.position() position.
        
        This method will not increment the position of the given ByteBuffer.
        
        In order to specify the offset into the ByteBuffer at which
        the matrix is stored, use .get3x3(int, ByteBuffer), taking
        the absolute position as parameter.

        Arguments
        - buffer: will receive the values of this matrix in column-major order at its current position

        Returns
        - the passed in buffer

        See
        - .get3x3(int, ByteBuffer)
        """
        ...


    def get3x3(self, index: int, buffer: "ByteBuffer") -> "ByteBuffer":
        """
        Store this matrix as an equivalent 3x3 matrix in column-major order into the supplied ByteBuffer starting at the specified
        absolute buffer position/index.
        
        This method will not increment the position of the given ByteBuffer.

        Arguments
        - index: the absolute position into the ByteBuffer
        - buffer: will receive the values of this matrix in column-major order

        Returns
        - the passed in buffer
        """
        ...


    def get4x4(self, buffer: "DoubleBuffer") -> "DoubleBuffer":
        """
        Store this matrix as an equivalent 4x4 matrix in column-major order into the supplied DoubleBuffer at the current
        buffer DoubleBuffer.position() position.
        
        This method will not increment the position of the given DoubleBuffer.
        
        In order to specify the offset into the DoubleBuffer at which
        the matrix is stored, use .get4x4(int, DoubleBuffer), taking
        the absolute position as parameter.

        Arguments
        - buffer: will receive the values of this matrix in column-major order at its current position

        Returns
        - the passed in buffer

        See
        - .get4x4(int, DoubleBuffer)
        """
        ...


    def get4x4(self, index: int, buffer: "DoubleBuffer") -> "DoubleBuffer":
        """
        Store this matrix as an equivalent 4x4 matrix in column-major order into the supplied DoubleBuffer starting at the specified
        absolute buffer position/index.
        
        This method will not increment the position of the given DoubleBuffer.

        Arguments
        - index: the absolute position into the DoubleBuffer
        - buffer: will receive the values of this matrix in column-major order

        Returns
        - the passed in buffer
        """
        ...


    def get4x4(self, buffer: "ByteBuffer") -> "ByteBuffer":
        """
        Store this matrix as an equivalent 4x4 matrix in column-major order into the supplied ByteBuffer at the current
        buffer ByteBuffer.position() position.
        
        This method will not increment the position of the given ByteBuffer.
        
        In order to specify the offset into the ByteBuffer at which
        the matrix is stored, use .get4x4(int, ByteBuffer), taking
        the absolute position as parameter.

        Arguments
        - buffer: will receive the values of this matrix in column-major order at its current position

        Returns
        - the passed in buffer

        See
        - .get4x4(int, ByteBuffer)
        """
        ...


    def get4x4(self, index: int, buffer: "ByteBuffer") -> "ByteBuffer":
        """
        Store this matrix as an equivalent 4x4 matrix in column-major order into the supplied ByteBuffer starting at the specified
        absolute buffer position/index.
        
        This method will not increment the position of the given ByteBuffer.

        Arguments
        - index: the absolute position into the ByteBuffer
        - buffer: will receive the values of this matrix in column-major order

        Returns
        - the passed in buffer
        """
        ...


    def getTransposed(self, buffer: "DoubleBuffer") -> "DoubleBuffer":
        ...


    def getTransposed(self, index: int, buffer: "DoubleBuffer") -> "DoubleBuffer":
        ...


    def getTransposed(self, buffer: "ByteBuffer") -> "ByteBuffer":
        ...


    def getTransposed(self, index: int, buffer: "ByteBuffer") -> "ByteBuffer":
        ...


    def getTransposed(self, buffer: "FloatBuffer") -> "FloatBuffer":
        ...


    def getTransposed(self, index: int, buffer: "FloatBuffer") -> "FloatBuffer":
        ...


    def getTransposedFloats(self, buffer: "ByteBuffer") -> "ByteBuffer":
        ...


    def getTransposedFloats(self, index: int, buffer: "ByteBuffer") -> "ByteBuffer":
        ...


    def getToAddress(self, address: int) -> "Matrix3x2dc":
        ...


    def getTransposedToAddress(self, address: int) -> "Matrix3x2dc":
        ...


    def get(self, arr: list[float], offset: int) -> list[float]:
        """
        Store this matrix into the supplied double array in column-major order at the given offset.

        Arguments
        - arr: the array to write the matrix values into
        - offset: the offset into the array

        Returns
        - the passed in array
        """
        ...


    def get(self, arr: list[float]) -> list[float]:
        """
        Store this matrix into the supplied double array in column-major order.
        
        In order to specify an explicit offset into the array, use the method .get(double[], int).

        Arguments
        - arr: the array to write the matrix values into

        Returns
        - the passed in array

        See
        - .get(double[], int)
        """
        ...


    def get3x3(self, arr: list[float], offset: int) -> list[float]:
        """
        Store this matrix as an equivalent 3x3 matrix in column-major order into the supplied float array at the given offset.

        Arguments
        - arr: the array to write the matrix values into
        - offset: the offset into the array

        Returns
        - the passed in array
        """
        ...


    def get3x3(self, arr: list[float]) -> list[float]:
        """
        Store this matrix as an equivalent 3x3 matrix in column-major order into the supplied float array.
        
        In order to specify an explicit offset into the array, use the method .get3x3(double[], int).

        Arguments
        - arr: the array to write the matrix values into

        Returns
        - the passed in array

        See
        - .get3x3(double[], int)
        """
        ...


    def get4x4(self, arr: list[float], offset: int) -> list[float]:
        """
        Store this matrix as an equivalent 4x4 matrix in column-major order into the supplied float array at the given offset.

        Arguments
        - arr: the array to write the matrix values into
        - offset: the offset into the array

        Returns
        - the passed in array
        """
        ...


    def get4x4(self, arr: list[float]) -> list[float]:
        """
        Store this matrix as an equivalent 4x4 matrix in column-major order into the supplied float array.
        
        In order to specify an explicit offset into the array, use the method .get4x4(double[], int).

        Arguments
        - arr: the array to write the matrix values into

        Returns
        - the passed in array

        See
        - .get4x4(double[], int)
        """
        ...


    def set(self, buffer: "DoubleBuffer") -> "Matrix3x2d":
        """
        Set the values of this matrix by reading 6 double values from the given DoubleBuffer in column-major order,
        starting at its current position.
        
        The DoubleBuffer is expected to contain the values in column-major order.
        
        The position of the DoubleBuffer will not be changed by this method.

        Arguments
        - buffer: the DoubleBuffer to read the matrix values from in column-major order

        Returns
        - this
        """
        ...


    def set(self, buffer: "ByteBuffer") -> "Matrix3x2d":
        """
        Set the values of this matrix by reading 6 double values from the given ByteBuffer in column-major order,
        starting at its current position.
        
        The ByteBuffer is expected to contain the values in column-major order.
        
        The position of the ByteBuffer will not be changed by this method.

        Arguments
        - buffer: the ByteBuffer to read the matrix values from in column-major order

        Returns
        - this
        """
        ...


    def set(self, index: int, buffer: "DoubleBuffer") -> "Matrix3x2d":
        """
        Set the values of this matrix by reading 6 double values from the given DoubleBuffer in column-major order,
        starting at the specified absolute buffer position/index.
        
        The DoubleBuffer is expected to contain the values in column-major order.
        
        The position of the DoubleBuffer will not be changed by this method.

        Arguments
        - index: the absolute position into the DoubleBuffer
        - buffer: the DoubleBuffer to read the matrix values from in column-major order

        Returns
        - this
        """
        ...


    def set(self, index: int, buffer: "ByteBuffer") -> "Matrix3x2d":
        """
        Set the values of this matrix by reading 6 double values from the given ByteBuffer in column-major order,
        starting at the specified absolute buffer position/index.
        
        The ByteBuffer is expected to contain the values in column-major order.
        
        The position of the ByteBuffer will not be changed by this method.

        Arguments
        - index: the absolute position into the ByteBuffer
        - buffer: the ByteBuffer to read the matrix values from in column-major order

        Returns
        - this
        """
        ...


    def setFromAddress(self, address: int) -> "Matrix3x2d":
        """
        Set the values of this matrix by reading 6 double values from off-heap memory in column-major order,
        starting at the given address.
        
        This method will throw an UnsupportedOperationException when JOML is used with `-Djoml.nounsafe`.
        
        *This method is unsafe as it can result in a crash of the JVM process when the specified address range does not belong to this process.*

        Arguments
        - address: the off-heap memory address to read the matrix values from in column-major order

        Returns
        - this
        """
        ...


    def setTransposedFromAddress(self, address: int) -> "Matrix3x2d":
        """
        Set the values of this matrix by reading 6 double values from off-heap memory in row-major order,
        starting at the given address.
        
        This method will throw an UnsupportedOperationException when JOML is used with `-Djoml.nounsafe`.
        
        *This method is unsafe as it can result in a crash of the JVM process when the specified address range does not belong to this process.*

        Arguments
        - address: the off-heap memory address to read the matrix values from in row-major order

        Returns
        - this
        """
        ...


    def zero(self) -> "Matrix3x2d":
        """
        Set all values within this matrix to zero.

        Returns
        - this
        """
        ...


    def identity(self) -> "Matrix3x2d":
        """
        Set this matrix to the identity.

        Returns
        - this
        """
        ...


    def scale(self, x: float, y: float, dest: "Matrix3x2d") -> "Matrix3x2d":
        """
        Apply scaling to this matrix by scaling the unit axes by the given x and y and store the result in `dest`.
        
        If `M` is `this` matrix and `S` the scaling matrix,
        then the new matrix will be `M * S`. So when transforming a
        vector `v` with the new matrix by using `M * S * v`, the scaling will be applied first!

        Arguments
        - x: the factor of the x component
        - y: the factor of the y component
        - dest: will hold the result

        Returns
        - dest
        """
        ...


    def scale(self, x: float, y: float) -> "Matrix3x2d":
        """
        Apply scaling to this matrix by scaling the base axes by the given x and y factors.
        
        If `M` is `this` matrix and `S` the scaling matrix,
        then the new matrix will be `M * S`. So when transforming a
        vector `v` with the new matrix by using `M * S * v`, the scaling will be applied first!

        Arguments
        - x: the factor of the x component
        - y: the factor of the y component

        Returns
        - this
        """
        ...


    def scale(self, xy: "Vector2dc") -> "Matrix3x2d":
        """
        Apply scaling to this matrix by scaling the base axes by the given `xy` factors.
        
        If `M` is `this` matrix and `S` the scaling matrix,
        then the new matrix will be `M * S`. So when transforming a
        vector `v` with the new matrix by using `M * S * v`, the scaling will be applied first!

        Arguments
        - xy: the factors of the x and y component, respectively

        Returns
        - this
        """
        ...


    def scale(self, xy: "Vector2dc", dest: "Matrix3x2d") -> "Matrix3x2d":
        """
        Apply scaling to this matrix by scaling the base axes by the given `xy` factors
        and store the result in `dest`.
        
        If `M` is `this` matrix and `S` the scaling matrix,
        then the new matrix will be `M * S`. So when transforming a
        vector `v` with the new matrix by using `M * S * v`, the scaling will be applied first!

        Arguments
        - xy: the factors of the x and y component, respectively
        - dest: will hold the result

        Returns
        - dest
        """
        ...


    def scale(self, xy: "Vector2fc") -> "Matrix3x2d":
        """
        Apply scaling to this matrix by scaling the base axes by the given `xy` factors.
        
        If `M` is `this` matrix and `S` the scaling matrix,
        then the new matrix will be `M * S`. So when transforming a
        vector `v` with the new matrix by using `M * S * v`, the scaling will be applied first!

        Arguments
        - xy: the factors of the x and y component, respectively

        Returns
        - this
        """
        ...


    def scale(self, xy: "Vector2fc", dest: "Matrix3x2d") -> "Matrix3x2d":
        """
        Apply scaling to this matrix by scaling the base axes by the given `xy` factors
        and store the result in `dest`.
        
        If `M` is `this` matrix and `S` the scaling matrix,
        then the new matrix will be `M * S`. So when transforming a
        vector `v` with the new matrix by using `M * S * v`, the scaling will be applied first!

        Arguments
        - xy: the factors of the x and y component, respectively
        - dest: will hold the result

        Returns
        - dest
        """
        ...


    def scale(self, xy: float, dest: "Matrix3x2d") -> "Matrix3x2d":
        """
        Apply scaling to this matrix by uniformly scaling the two base axes by the given `xy` factor
        and store the result in `dest`.
        
        If `M` is `this` matrix and `S` the scaling matrix,
        then the new matrix will be `M * S`. So when transforming a
        vector `v` with the new matrix by using `M * S * v`, the scaling will be applied first!

        Arguments
        - xy: the factor for the two components
        - dest: will hold the result

        Returns
        - dest

        See
        - .scale(double, double, Matrix3x2d)
        """
        ...


    def scale(self, xy: float) -> "Matrix3x2d":
        """
        Apply scaling to this matrix by uniformly scaling the two base axes by the given `xyz` factor.
        
        If `M` is `this` matrix and `S` the scaling matrix,
        then the new matrix will be `M * S`. So when transforming a
        vector `v` with the new matrix by using `M * S * v`, the scaling will be applied first!

        Arguments
        - xy: the factor for the two components

        Returns
        - this

        See
        - .scale(double, double)
        """
        ...


    def scaleLocal(self, x: float, y: float, dest: "Matrix3x2d") -> "Matrix3x2d":
        ...


    def scaleLocal(self, x: float, y: float) -> "Matrix3x2d":
        """
        Pre-multiply scaling to this matrix by scaling the base axes by the given x and y factors.
        
        If `M` is `this` matrix and `S` the scaling matrix,
        then the new matrix will be `S * M`. So when transforming a
        vector `v` with the new matrix by using `S * M * v`, the
        scaling will be applied last!

        Arguments
        - x: the factor of the x component
        - y: the factor of the y component

        Returns
        - this
        """
        ...


    def scaleLocal(self, xy: float, dest: "Matrix3x2d") -> "Matrix3x2d":
        ...


    def scaleLocal(self, xy: float) -> "Matrix3x2d":
        """
        Pre-multiply scaling to this matrix by scaling the base axes by the given xy factor.
        
        If `M` is `this` matrix and `S` the scaling matrix,
        then the new matrix will be `S * M`. So when transforming a
        vector `v` with the new matrix by using `S * M * v`, the
        scaling will be applied last!

        Arguments
        - xy: the factor of the x and y component

        Returns
        - this
        """
        ...


    def scaleAround(self, sx: float, sy: float, ox: float, oy: float, dest: "Matrix3x2d") -> "Matrix3x2d":
        """
        Apply scaling to `this` matrix by scaling the base axes by the given sx and
        sy factors while using `(ox, oy)` as the scaling origin, and store the result in `dest`.
        
        If `M` is `this` matrix and `S` the scaling matrix,
        then the new matrix will be `M * S`. So when transforming a
        vector `v` with the new matrix by using `M * S * v`
        , the scaling will be applied first!
        
        This method is equivalent to calling: `translate(ox, oy, dest).scale(sx, sy).translate(-ox, -oy)`

        Arguments
        - sx: the scaling factor of the x component
        - sy: the scaling factor of the y component
        - ox: the x coordinate of the scaling origin
        - oy: the y coordinate of the scaling origin
        - dest: will hold the result

        Returns
        - dest
        """
        ...


    def scaleAround(self, sx: float, sy: float, ox: float, oy: float) -> "Matrix3x2d":
        """
        Apply scaling to this matrix by scaling the base axes by the given sx and
        sy factors while using `(ox, oy)` as the scaling origin.
        
        If `M` is `this` matrix and `S` the scaling matrix,
        then the new matrix will be `M * S`. So when transforming a
        vector `v` with the new matrix by using `M * S * v`, the
        scaling will be applied first!
        
        This method is equivalent to calling: `translate(ox, oy).scale(sx, sy).translate(-ox, -oy)`

        Arguments
        - sx: the scaling factor of the x component
        - sy: the scaling factor of the y component
        - ox: the x coordinate of the scaling origin
        - oy: the y coordinate of the scaling origin

        Returns
        - this
        """
        ...


    def scaleAround(self, factor: float, ox: float, oy: float, dest: "Matrix3x2d") -> "Matrix3x2d":
        """
        Apply scaling to this matrix by scaling the base axes by the given `factor`
        while using `(ox, oy)` as the scaling origin,
        and store the result in `dest`.
        
        If `M` is `this` matrix and `S` the scaling matrix,
        then the new matrix will be `M * S`. So when transforming a
        vector `v` with the new matrix by using `M * S * v`, the
        scaling will be applied first!
        
        This method is equivalent to calling: `translate(ox, oy, dest).scale(factor).translate(-ox, -oy)`

        Arguments
        - factor: the scaling factor for all three axes
        - ox: the x coordinate of the scaling origin
        - oy: the y coordinate of the scaling origin
        - dest: will hold the result

        Returns
        - this
        """
        ...


    def scaleAround(self, factor: float, ox: float, oy: float) -> "Matrix3x2d":
        """
        Apply scaling to this matrix by scaling the base axes by the given `factor`
        while using `(ox, oy)` as the scaling origin.
        
        If `M` is `this` matrix and `S` the scaling matrix,
        then the new matrix will be `M * S`. So when transforming a
        vector `v` with the new matrix by using `M * S * v`, the
        scaling will be applied first!
        
        This method is equivalent to calling: `translate(ox, oy).scale(factor).translate(-ox, -oy)`

        Arguments
        - factor: the scaling factor for all axes
        - ox: the x coordinate of the scaling origin
        - oy: the y coordinate of the scaling origin

        Returns
        - this
        """
        ...


    def scaleAroundLocal(self, sx: float, sy: float, ox: float, oy: float, dest: "Matrix3x2d") -> "Matrix3x2d":
        ...


    def scaleAroundLocal(self, factor: float, ox: float, oy: float, dest: "Matrix3x2d") -> "Matrix3x2d":
        ...


    def scaleAroundLocal(self, sx: float, sy: float, sz: float, ox: float, oy: float, oz: float) -> "Matrix3x2d":
        """
        Pre-multiply scaling to this matrix by scaling the base axes by the given sx and
        sy factors while using `(ox, oy)` as the scaling origin.
        
        If `M` is `this` matrix and `S` the scaling matrix,
        then the new matrix will be `S * M`. So when transforming a
        vector `v` with the new matrix by using `S * M * v`, the
        scaling will be applied last!
        
        This method is equivalent to calling: `new Matrix3x2d().translate(ox, oy).scale(sx, sy).translate(-ox, -oy).mul(this, this)`

        Arguments
        - sx: the scaling factor of the x component
        - sy: the scaling factor of the y component
        - sz: the scaling factor of the z component
        - ox: the x coordinate of the scaling origin
        - oy: the y coordinate of the scaling origin
        - oz: the z coordinate of the scaling origin

        Returns
        - this
        """
        ...


    def scaleAroundLocal(self, factor: float, ox: float, oy: float) -> "Matrix3x2d":
        """
        Pre-multiply scaling to this matrix by scaling the base axes by the given `factor`
        while using `(ox, oy)` as the scaling origin.
        
        If `M` is `this` matrix and `S` the scaling matrix,
        then the new matrix will be `S * M`. So when transforming a
        vector `v` with the new matrix by using `S * M * v`, the
        scaling will be applied last!
        
        This method is equivalent to calling: `new Matrix3x2d().translate(ox, oy).scale(factor).translate(-ox, -oy).mul(this, this)`

        Arguments
        - factor: the scaling factor for all three axes
        - ox: the x coordinate of the scaling origin
        - oy: the y coordinate of the scaling origin

        Returns
        - this
        """
        ...


    def scaling(self, factor: float) -> "Matrix3x2d":
        """
        Set this matrix to be a simple scale matrix, which scales the two base axes uniformly by the given factor.
        
        The resulting matrix can be multiplied against another transformation
        matrix to obtain an additional scaling.
        
        In order to post-multiply a scaling transformation directly to a matrix, use .scale(double) scale() instead.

        Arguments
        - factor: the scale factor in x and y

        Returns
        - this

        See
        - .scale(double)
        """
        ...


    def scaling(self, x: float, y: float) -> "Matrix3x2d":
        """
        Set this matrix to be a simple scale matrix.

        Arguments
        - x: the scale in x
        - y: the scale in y

        Returns
        - this
        """
        ...


    def rotation(self, angle: float) -> "Matrix3x2d":
        """
        Set this matrix to a rotation matrix which rotates the given radians.
        
        The resulting matrix can be multiplied against another transformation
        matrix to obtain an additional rotation.
        
        In order to apply the rotation transformation to an existing transformation,
        use .rotate(double) rotate() instead.

        Arguments
        - angle: the angle in radians

        Returns
        - this

        See
        - .rotate(double)
        """
        ...


    def transform(self, v: "Vector3d") -> "Vector3d":
        """
        Transform/multiply the given vector by this matrix by assuming a third row in this matrix of `(0, 0, 1)`
        and store the result in that vector.

        Arguments
        - v: the vector to transform and to hold the final result

        Returns
        - v

        See
        - Vector3d.mul(Matrix3x2dc)
        """
        ...


    def transform(self, v: "Vector3dc", dest: "Vector3d") -> "Vector3d":
        """
        Transform/multiply the given vector by this matrix by assuming a third row in this matrix of `(0, 0, 1)`
        and store the result in `dest`.

        Arguments
        - v: the vector to transform
        - dest: will contain the result

        Returns
        - dest

        See
        - Vector3d.mul(Matrix3x2dc, Vector3d)
        """
        ...


    def transform(self, x: float, y: float, z: float, dest: "Vector3d") -> "Vector3d":
        """
        Transform/multiply the given vector `(x, y, z)` by this matrix and store the result in `dest`.

        Arguments
        - x: the x component of the vector to transform
        - y: the y component of the vector to transform
        - z: the z component of the vector to transform
        - dest: will contain the result

        Returns
        - dest
        """
        ...


    def transformPosition(self, v: "Vector2d") -> "Vector2d":
        """
        Transform/multiply the given 2D-vector, as if it was a 3D-vector with z=1, by
        this matrix and store the result in that vector.
        
        The given 2D-vector is treated as a 3D-vector with its z-component being 1.0, so it
        will represent a position/location in 2D-space rather than a direction.
        
        In order to store the result in another vector, use .transformPosition(Vector2dc, Vector2d).

        Arguments
        - v: the vector to transform and to hold the final result

        Returns
        - v

        See
        - .transform(Vector3d)
        """
        ...


    def transformPosition(self, v: "Vector2dc", dest: "Vector2d") -> "Vector2d":
        """
        Transform/multiply the given 2D-vector, as if it was a 3D-vector with z=1, by
        this matrix and store the result in `dest`.
        
        The given 2D-vector is treated as a 3D-vector with its z-component being 1.0, so it
        will represent a position/location in 2D-space rather than a direction.
        
        In order to store the result in the same vector, use .transformPosition(Vector2d).

        Arguments
        - v: the vector to transform
        - dest: will hold the result

        Returns
        - dest

        See
        - .transform(Vector3dc, Vector3d)
        """
        ...


    def transformPosition(self, x: float, y: float, dest: "Vector2d") -> "Vector2d":
        """
        Transform/multiply the given 2D-vector `(x, y)`, as if it was a 3D-vector with z=1, by
        this matrix and store the result in `dest`.
        
        The given 2D-vector is treated as a 3D-vector with its z-component being 1.0, so it
        will represent a position/location in 2D-space rather than a direction.
        
        In order to store the result in the same vector, use .transformPosition(Vector2d).

        Arguments
        - x: the x component of the vector to transform
        - y: the y component of the vector to transform
        - dest: will hold the result

        Returns
        - dest

        See
        - .transform(Vector3dc, Vector3d)
        """
        ...


    def transformDirection(self, v: "Vector2d") -> "Vector2d":
        """
        Transform/multiply the given 2D-vector, as if it was a 3D-vector with z=0, by
        this matrix and store the result in that vector.
        
        The given 2D-vector is treated as a 3D-vector with its z-component being `0.0`, so it
        will represent a direction in 2D-space rather than a position. This method will therefore
        not take the translation part of the matrix into account.
        
        In order to store the result in another vector, use .transformDirection(Vector2dc, Vector2d).

        Arguments
        - v: the vector to transform and to hold the final result

        Returns
        - v

        See
        - .transformDirection(Vector2dc, Vector2d)
        """
        ...


    def transformDirection(self, v: "Vector2dc", dest: "Vector2d") -> "Vector2d":
        """
        Transform/multiply the given 2D-vector, as if it was a 3D-vector with z=0, by
        this matrix and store the result in `dest`.
        
        The given 2D-vector is treated as a 3D-vector with its z-component being `0.0`, so it
        will represent a direction in 2D-space rather than a position. This method will therefore
        not take the translation part of the matrix into account.
        
        In order to store the result in the same vector, use .transformDirection(Vector2d).

        Arguments
        - v: the vector to transform and to hold the final result
        - dest: will hold the result

        Returns
        - dest

        See
        - .transformDirection(Vector2d)
        """
        ...


    def transformDirection(self, x: float, y: float, dest: "Vector2d") -> "Vector2d":
        """
        Transform/multiply the given 2D-vector `(x, y)`, as if it was a 3D-vector with z=0, by
        this matrix and store the result in `dest`.
        
        The given 2D-vector is treated as a 3D-vector with its z-component being `0.0`, so it
        will represent a direction in 2D-space rather than a position. This method will therefore
        not take the translation part of the matrix into account.
        
        In order to store the result in the same vector, use .transformDirection(Vector2d).

        Arguments
        - x: the x component of the vector to transform
        - y: the y component of the vector to transform
        - dest: will hold the result

        Returns
        - dest

        See
        - .transformDirection(Vector2d)
        """
        ...


    def writeExternal(self, out: "ObjectOutput") -> None:
        ...


    def readExternal(self, in: "ObjectInput") -> None:
        ...


    def rotate(self, ang: float) -> "Matrix3x2d":
        """
        Apply a rotation transformation to this matrix by rotating the given amount of radians.
        
        If `M` is `this` matrix and `R` the rotation matrix,
        then the new matrix will be `M * R`. So when transforming a
        vector `v` with the new matrix by using `M * R * v`
        , the rotation will be applied first!

        Arguments
        - ang: the angle in radians

        Returns
        - this
        """
        ...


    def rotate(self, ang: float, dest: "Matrix3x2d") -> "Matrix3x2d":
        """
        Apply a rotation transformation to this matrix by rotating the given amount of radians and store the result in `dest`.
        
        If `M` is `this` matrix and `R` the rotation matrix,
        then the new matrix will be `M * R`. So when transforming a
        vector `v` with the new matrix by using `M * R * v`, the rotation will be applied first!

        Arguments
        - ang: the angle in radians
        - dest: will hold the result

        Returns
        - dest
        """
        ...


    def rotateLocal(self, ang: float, dest: "Matrix3x2d") -> "Matrix3x2d":
        """
        Pre-multiply a rotation to this matrix by rotating the given amount of radians and store the result in `dest`.
        
        If `M` is `this` matrix and `R` the rotation matrix,
        then the new matrix will be `R * M`. So when transforming a
        vector `v` with the new matrix by using `R * M * v`, the
        rotation will be applied last!
        
        In order to set the matrix to a rotation matrix without pre-multiplying the rotation
        transformation, use .rotation(double) rotation().
        
        Reference: <a href="http://en.wikipedia.org/wiki/Rotation_matrix#Rotation_matrix_from_axis_and_angle">http://en.wikipedia.org</a>

        Arguments
        - ang: the angle in radians to rotate
        - dest: will hold the result

        Returns
        - dest

        See
        - .rotation(double)
        """
        ...


    def rotateLocal(self, ang: float) -> "Matrix3x2d":
        """
        Pre-multiply a rotation to this matrix by rotating the given amount of radians.
        
        If `M` is `this` matrix and `R` the rotation matrix,
        then the new matrix will be `R * M`. So when transforming a
        vector `v` with the new matrix by using `R * M * v`, the
        rotation will be applied last!
        
        In order to set the matrix to a rotation matrix without pre-multiplying the rotation
        transformation, use .rotation(double) rotation().
        
        Reference: <a href="http://en.wikipedia.org/wiki/Rotation_matrix#Rotation_matrix_from_axis_and_angle">http://en.wikipedia.org</a>

        Arguments
        - ang: the angle in radians to rotate

        Returns
        - this

        See
        - .rotation(double)
        """
        ...


    def rotateAbout(self, ang: float, x: float, y: float) -> "Matrix3x2d":
        """
        Apply a rotation transformation to this matrix by rotating the given amount of radians about
        the specified rotation center `(x, y)`.
        
        This method is equivalent to calling: `translate(x, y).rotate(ang).translate(-x, -y)`
        
        If `M` is `this` matrix and `R` the rotation matrix,
        then the new matrix will be `M * R`. So when transforming a
        vector `v` with the new matrix by using `M * R * v`, the rotation will be applied first!

        Arguments
        - ang: the angle in radians
        - x: the x component of the rotation center
        - y: the y component of the rotation center

        Returns
        - this

        See
        - .rotate(double)
        """
        ...


    def rotateAbout(self, ang: float, x: float, y: float, dest: "Matrix3x2d") -> "Matrix3x2d":
        """
        Apply a rotation transformation to this matrix by rotating the given amount of radians about
        the specified rotation center `(x, y)` and store the result in `dest`.
        
        This method is equivalent to calling: `translate(x, y, dest).rotate(ang).translate(-x, -y)`
        
        If `M` is `this` matrix and `R` the rotation matrix,
        then the new matrix will be `M * R`. So when transforming a
        vector `v` with the new matrix by using `M * R * v`, the rotation will be applied first!

        Arguments
        - ang: the angle in radians
        - x: the x component of the rotation center
        - y: the y component of the rotation center
        - dest: will hold the result

        Returns
        - dest

        See
        - .rotate(double, Matrix3x2d)
        """
        ...


    def rotateTo(self, fromDir: "Vector2dc", toDir: "Vector2dc", dest: "Matrix3x2d") -> "Matrix3x2d":
        """
        Apply a rotation transformation to this matrix that rotates the given normalized `fromDir` direction vector
        to point along the normalized `toDir`, and store the result in `dest`.
        
        If `M` is `this` matrix and `R` the rotation matrix,
        then the new matrix will be `M * R`. So when transforming a
        vector `v` with the new matrix by using `M * R * v`, the rotation will be applied first!

        Arguments
        - fromDir: the normalized direction which should be rotate to point along `toDir`
        - toDir: the normalized destination direction
        - dest: will hold the result

        Returns
        - dest
        """
        ...


    def rotateTo(self, fromDir: "Vector2dc", toDir: "Vector2dc") -> "Matrix3x2d":
        """
        Apply a rotation transformation to this matrix that rotates the given normalized `fromDir` direction vector
        to point along the normalized `toDir`.
        
        If `M` is `this` matrix and `R` the rotation matrix,
        then the new matrix will be `M * R`. So when transforming a
        vector `v` with the new matrix by using `M * R * v`, the rotation will be applied first!

        Arguments
        - fromDir: the normalized direction which should be rotate to point along `toDir`
        - toDir: the normalized destination direction

        Returns
        - this
        """
        ...


    def view(self, left: float, right: float, bottom: float, top: float, dest: "Matrix3x2d") -> "Matrix3x2d":
        """
        Apply a "view" transformation to this matrix that maps the given `(left, bottom)` and
        `(right, top)` corners to `(-1, -1)` and `(1, 1)` respectively and store the result in `dest`.
        
        If `M` is `this` matrix and `O` the orthographic projection matrix,
        then the new matrix will be `M * O`. So when transforming a
        vector `v` with the new matrix by using `M * O * v`, the
        orthographic projection transformation will be applied first!

        Arguments
        - left: the distance from the center to the left view edge
        - right: the distance from the center to the right view edge
        - bottom: the distance from the center to the bottom view edge
        - top: the distance from the center to the top view edge
        - dest: will hold the result

        Returns
        - dest

        See
        - .setView(double, double, double, double)
        """
        ...


    def view(self, left: float, right: float, bottom: float, top: float) -> "Matrix3x2d":
        """
        Apply a "view" transformation to this matrix that maps the given `(left, bottom)` and
        `(right, top)` corners to `(-1, -1)` and `(1, 1)` respectively.
        
        If `M` is `this` matrix and `O` the orthographic projection matrix,
        then the new matrix will be `M * O`. So when transforming a
        vector `v` with the new matrix by using `M * O * v`, the
        orthographic projection transformation will be applied first!

        Arguments
        - left: the distance from the center to the left view edge
        - right: the distance from the center to the right view edge
        - bottom: the distance from the center to the bottom view edge
        - top: the distance from the center to the top view edge

        Returns
        - this

        See
        - .setView(double, double, double, double)
        """
        ...


    def setView(self, left: float, right: float, bottom: float, top: float) -> "Matrix3x2d":
        """
        Set this matrix to define a "view" transformation that maps the given `(left, bottom)` and
        `(right, top)` corners to `(-1, -1)` and `(1, 1)` respectively.

        Arguments
        - left: the distance from the center to the left view edge
        - right: the distance from the center to the right view edge
        - bottom: the distance from the center to the bottom view edge
        - top: the distance from the center to the top view edge

        Returns
        - this

        See
        - .view(double, double, double, double)
        """
        ...


    def origin(self, origin: "Vector2d") -> "Vector2d":
        """
        Obtain the position that gets transformed to the origin by `this` matrix.
        This can be used to get the position of the "camera" from a given *view* transformation matrix.
        
        This method is equivalent to the following code:
        ```
        Matrix3x2d inv = new Matrix3x2d(this).invert();
        inv.transform(origin.set(0, 0));
        ```

        Arguments
        - origin: will hold the position transformed to the origin

        Returns
        - origin
        """
        ...


    def viewArea(self, area: list[float]) -> list[float]:
        """
        Obtain the extents of the view transformation of `this` matrix and store it in `area`.
        This can be used to determine which region of the screen (i.e. the NDC space) is covered by the view.

        Arguments
        - area: will hold the view area as `[minX, minY, maxX, maxY]`

        Returns
        - area
        """
        ...


    def positiveX(self, dir: "Vector2d") -> "Vector2d":
        ...


    def normalizedPositiveX(self, dir: "Vector2d") -> "Vector2d":
        ...


    def positiveY(self, dir: "Vector2d") -> "Vector2d":
        ...


    def normalizedPositiveY(self, dir: "Vector2d") -> "Vector2d":
        ...


    def unproject(self, winX: float, winY: float, viewport: list[int], dest: "Vector2d") -> "Vector2d":
        """
        Unproject the given window coordinates `(winX, winY)` by `this` matrix using the specified viewport.
        
        This method first converts the given window coordinates to normalized device coordinates in the range `[-1..1]`
        and then transforms those NDC coordinates by the inverse of `this` matrix.  
        
        As a necessary computation step for unprojecting, this method computes the inverse of `this` matrix.
        In order to avoid computing the matrix inverse with every invocation, the inverse of `this` matrix can be built
        once outside using .invert(Matrix3x2d) and then the method .unprojectInv(double, double, int[], Vector2d) unprojectInv() can be invoked on it.

        Arguments
        - winX: the x-coordinate in window coordinates (pixels)
        - winY: the y-coordinate in window coordinates (pixels)
        - viewport: the viewport described by `[x, y, width, height]`
        - dest: will hold the unprojected position

        Returns
        - dest

        See
        - .invert(Matrix3x2d)
        """
        ...


    def unprojectInv(self, winX: float, winY: float, viewport: list[int], dest: "Vector2d") -> "Vector2d":
        """
        Unproject the given window coordinates `(winX, winY)` by `this` matrix using the specified viewport.
        
        This method differs from .unproject(double, double, int[], Vector2d) unproject() 
        in that it assumes that `this` is already the inverse matrix of the original projection matrix.
        It exists to avoid recomputing the matrix inverse with every invocation.

        Arguments
        - winX: the x-coordinate in window coordinates (pixels)
        - winY: the y-coordinate in window coordinates (pixels)
        - viewport: the viewport described by `[x, y, width, height]`
        - dest: will hold the unprojected position

        Returns
        - dest

        See
        - .unproject(double, double, int[], Vector2d)
        """
        ...


    def span(self, corner: "Vector2d", xDir: "Vector2d", yDir: "Vector2d") -> "Matrix3x2d":
        """
        Compute the extents of the coordinate system before this transformation was applied and store the resulting
        corner coordinates in `corner` and the span vectors in `xDir` and `yDir`.
        
        That means, given the maximum extents of the coordinate system between `[-1..+1]` in all dimensions,
        this method returns one corner and the length and direction of the two base axis vectors in the coordinate
        system before this transformation is applied, which transforms into the corner coordinates `[-1, +1]`.

        Arguments
        - corner: will hold one corner of the span
        - xDir: will hold the direction and length of the span along the positive X axis
        - yDir: will hold the direction and length of the span along the positive Y axis

        Returns
        - this
        """
        ...


    def testPoint(self, x: float, y: float) -> bool:
        ...


    def testCircle(self, x: float, y: float, r: float) -> bool:
        ...


    def testAar(self, minX: float, minY: float, maxX: float, maxY: float) -> bool:
        ...


    def hashCode(self) -> int:
        ...


    def equals(self, obj: "Object") -> bool:
        ...


    def equals(self, m: "Matrix3x2dc", delta: float) -> bool:
        ...


    def isFinite(self) -> bool:
        ...


    def clone(self) -> "Object":
        ...



"""
Python module generated from Java source file org.bukkit.entity.BlockDisplay

Java source file obtained from artifact spigot-api version 1.21.4-R0.1-20250303.102353-42

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from org.bukkit.block.data import BlockData
from org.bukkit.entity import *
from typing import Any, Callable, Iterable, Tuple


class BlockDisplay(Display):
    """
    Represents a block display entity.
    """

    def getBlock(self) -> "BlockData":
        """
        Gets the displayed block.

        Returns
        - the displayed block
        """
        ...


    def setBlock(self, block: "BlockData") -> None:
        """
        Sets the displayed block.

        Arguments
        - block: the new block
        """
        ...



"""
Python module generated from Java source file net.md_5.bungee.api.chat.ScoreComponent

Java source file obtained from artifact bungeecord-chat version 1.20-R0.2-20240119.213604-65

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from net.md_5.bungee.api.chat import *
from typing import Any, Callable, Iterable, Tuple


class ScoreComponent(BaseComponent):
    """
    This component displays the score based on a player score on the scoreboard.
    
    The **name** is the name of the player stored on the scoreboard, which may
    be a "fake" player. It can also be a target selector that **must** resolve
    to 1 target, and may target non-player entities.
    
    With a book, /tellraw, or /title, using the wildcard '*' in the place of a
    name or target selector will cause all players to see their own score in the
    specified objective.
    
    **Signs cannot use the '*' wildcard**
    
    These values are filled in by the server-side implementation.
    
    As of 1.12.2, a bug ( MC-56373 ) prevents full usage within hover events.
    """

    def __init__(self, name: str, objective: str):
        """
        Creates a new score component with the specified name and objective.
        If not specifically set, value will default to an empty string;
        signifying that the scoreboard value should take precedence. If not null,
        nor empty, `value` will override any value found in the
        scoreboard.
        The value defaults to an empty string.

        Arguments
        - name: the name of the entity, or an entity selector, whose score
        should be displayed
        - objective: the internal name of the objective the entity's score is
        attached to
        """
        ...


    def __init__(self, original: "ScoreComponent"):
        """
        Creates a score component from the original to clone it.

        Arguments
        - original: the original for the new score component
        """
        ...


    def duplicate(self) -> "ScoreComponent":
        ...



"""
Python module generated from Java source file org.bukkit.entity.Projectile

Java source file obtained from artifact spigot-api version 1.21.4-R0.1-20250303.102353-42

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from org.bukkit.entity import *
from org.bukkit.projectiles import ProjectileSource
from typing import Any, Callable, Iterable, Tuple


class Projectile(Entity):
    """
    Represents a shootable entity.
    """

    def getShooter(self) -> "ProjectileSource":
        """
        Retrieve the shooter of this projectile.

        Returns
        - the ProjectileSource that shot this projectile
        """
        ...


    def setShooter(self, source: "ProjectileSource") -> None:
        """
        Set the shooter of this projectile.

        Arguments
        - source: the ProjectileSource that shot this projectile
        """
        ...


    def doesBounce(self) -> bool:
        """
        Determine if this projectile should bounce or not when it hits.

        Returns
        - True if it should bounce.

        Deprecated
        - does not do anything
        """
        ...


    def setBounce(self, doesBounce: bool) -> None:
        """
        Set whether or not this projectile should bounce or not when it hits
        something.

        Arguments
        - doesBounce: whether or not it should bounce.

        Deprecated
        - does not do anything
        """
        ...



"""
Python module generated from Java source file com.google.gson.JsonSerializationContext

Java source file obtained from artifact gson version 2.11.0

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from com.google.gson import *
from java.lang.reflect import Type
from typing import Any, Callable, Iterable, Tuple


class JsonSerializationContext:
    """
    Context for serialization that is passed to a custom serializer during invocation of its JsonSerializer.serialize(Object, Type, JsonSerializationContext) method.

    Author(s)
    - Joel Leitch
    """

    def serialize(self, src: "Object") -> "JsonElement":
        """
        Invokes default serialization on the specified object.

        Arguments
        - src: the object that needs to be serialized.

        Returns
        - a tree of JsonElements corresponding to the serialized form of `src`.
        """
        ...


    def serialize(self, src: "Object", typeOfSrc: "Type") -> "JsonElement":
        """
        Invokes default serialization on the specified object passing the specific type information. It
        should never be invoked on the element received as a parameter of the JsonSerializer.serialize(Object, Type, JsonSerializationContext) method. Doing so will result
        in an infinite loop since Gson will in-turn call the custom serializer again.

        Arguments
        - src: the object that needs to be serialized.
        - typeOfSrc: the actual genericized type of src object.

        Returns
        - a tree of JsonElements corresponding to the serialized form of `src`.
        """
        ...



"""
Python module generated from Java source file org.joml.Vector2d

Java source file obtained from artifact joml version 1.10.8

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from java.io import Externalizable
from java.io import IOException
from java.io import ObjectInput
from java.io import ObjectOutput
from java.text import DecimalFormat
from java.text import NumberFormat
from org.joml import *
from typing import Any, Callable, Iterable, Tuple


class Vector2d(Externalizable, Cloneable, Vector2dc):
    """
    Represents a 2D vector with double-precision.

    Author(s)
    - F. Neurath
    """

    def __init__(self):
        """
        Create a new Vector2d and initialize its components to zero.
        """
        ...


    def __init__(self, d: float):
        """
        Create a new Vector2d and initialize both of its components with the given value.

        Arguments
        - d: the value of both components
        """
        ...


    def __init__(self, x: float, y: float):
        """
        Create a new Vector2d and initialize its components to the given values.

        Arguments
        - x: the x value
        - y: the y value
        """
        ...


    def __init__(self, v: "Vector2dc"):
        """
        Create a new Vector2d and initialize its components to the one of the given vector.

        Arguments
        - v: the Vector2dc to copy the values from
        """
        ...


    def __init__(self, v: "Vector2fc"):
        """
        Create a new Vector2d and initialize its components to the one of the given vector.

        Arguments
        - v: the Vector2fc to copy the values from
        """
        ...


    def __init__(self, v: "Vector2ic"):
        """
        Create a new Vector2d and initialize its components to the one of the given vector.

        Arguments
        - v: the Vector2ic to copy the values from
        """
        ...


    def __init__(self, v: "Vector3dc"):
        """
        Create a new Vector2d and initialize its components using the `x` and `y`
        components of the provided vector.

        Arguments
        - v: the Vector3dc to copy the `x` and `y` components from
        """
        ...


    def __init__(self, v: "Vector3fc"):
        """
        Create a new Vector2d and initialize its components using the `x` and `y`
        components of the provided vector.

        Arguments
        - v: the Vector3fc to copy the `x` and `y` components from
        """
        ...


    def __init__(self, v: "Vector3ic"):
        """
        Create a new Vector2d and initialize its components using the `x` and `y`
        components of the provided vector.

        Arguments
        - v: the Vector3ic to copy the `x` and `y` components from
        """
        ...


    def __init__(self, xy: list[float]):
        """
        Create a new Vector2d and initialize its two components from the first
        two elements of the given array.

        Arguments
        - xy: the array containing at least three elements
        """
        ...


    def __init__(self, xy: list[float]):
        """
        Create a new Vector2d and initialize its two components from the first
        two elements of the given array.

        Arguments
        - xy: the array containing at least two elements
        """
        ...


    def __init__(self, buffer: "ByteBuffer"):
        """
        Create a new Vector2d and read this vector from the supplied ByteBuffer
        at the current buffer ByteBuffer.position() position.
        
        This method will not increment the position of the given ByteBuffer.
        
        In order to specify the offset into the ByteBuffer at which
        the vector is read, use .Vector2d(int, ByteBuffer), taking
        the absolute position as parameter.

        Arguments
        - buffer: values will be read in `x, y` order

        See
        - .Vector2d(int, ByteBuffer)
        """
        ...


    def __init__(self, index: int, buffer: "ByteBuffer"):
        """
        Create a new Vector2d and read this vector from the supplied ByteBuffer
        starting at the specified absolute buffer position/index.
        
        This method will not increment the position of the given ByteBuffer.

        Arguments
        - index: the absolute position into the ByteBuffer
        - buffer: values will be read in `x, y` order
        """
        ...


    def __init__(self, buffer: "DoubleBuffer"):
        """
        Create a new Vector2d and read this vector from the supplied DoubleBuffer
        at the current buffer DoubleBuffer.position() position.
        
        This method will not increment the position of the given DoubleBuffer.
        
        In order to specify the offset into the DoubleBuffer at which
        the vector is read, use .Vector2d(int, DoubleBuffer), taking
        the absolute position as parameter.

        Arguments
        - buffer: values will be read in `x, y` order

        See
        - .Vector2d(int, DoubleBuffer)
        """
        ...


    def __init__(self, index: int, buffer: "DoubleBuffer"):
        """
        Create a new Vector2d and read this vector from the supplied DoubleBuffer
        starting at the specified absolute buffer position/index.
        
        This method will not increment the position of the given DoubleBuffer.

        Arguments
        - index: the absolute position into the DoubleBuffer
        - buffer: values will be read in `x, y` order
        """
        ...


    def x(self) -> float:
        ...


    def y(self) -> float:
        ...


    def set(self, d: float) -> "Vector2d":
        """
        Set the x and y components to the supplied value.

        Arguments
        - d: the value of both components

        Returns
        - this
        """
        ...


    def set(self, x: float, y: float) -> "Vector2d":
        """
        Set the x and y components to the supplied values.

        Arguments
        - x: the x value
        - y: the y value

        Returns
        - this
        """
        ...


    def set(self, v: "Vector2dc") -> "Vector2d":
        """
        Set this vector to the values of v.

        Arguments
        - v: the vector to copy from

        Returns
        - this
        """
        ...


    def set(self, v: "Vector2fc") -> "Vector2d":
        """
        Set this vector to be a clone of `v`.

        Arguments
        - v: the vector to copy from

        Returns
        - this
        """
        ...


    def set(self, v: "Vector2ic") -> "Vector2d":
        """
        Set this vector to be a clone of `v`.

        Arguments
        - v: the vector to copy from

        Returns
        - this
        """
        ...


    def set(self, v: "Vector3dc") -> "Vector2d":
        """
        Set this vector to the `(x, y)` components of `v`.

        Arguments
        - v: the vector to copy from

        Returns
        - this
        """
        ...


    def set(self, v: "Vector3fc") -> "Vector2d":
        """
        Set this vector to the `(x, y)` components of `v`.

        Arguments
        - v: the vector to copy from

        Returns
        - this
        """
        ...


    def set(self, v: "Vector3ic") -> "Vector2d":
        """
        Set this vector to the `(x, y)` components of `v`.

        Arguments
        - v: the vector to copy from

        Returns
        - this
        """
        ...


    def set(self, xy: list[float]) -> "Vector2d":
        """
        Set the two components of this vector to the first two elements of the given array.

        Arguments
        - xy: the array containing at least three elements

        Returns
        - this
        """
        ...


    def set(self, xy: list[float]) -> "Vector2d":
        """
        Set the two components of this vector to the first two elements of the given array.

        Arguments
        - xy: the array containing at least two elements

        Returns
        - this
        """
        ...


    def set(self, buffer: "ByteBuffer") -> "Vector2d":
        """
        Read this vector from the supplied ByteBuffer at the current
        buffer ByteBuffer.position() position.
        
        This method will not increment the position of the given ByteBuffer.
        
        In order to specify the offset into the ByteBuffer at which
        the vector is read, use .set(int, ByteBuffer), taking
        the absolute position as parameter.

        Arguments
        - buffer: values will be read in `x, y` order

        Returns
        - this

        See
        - .set(int, ByteBuffer)
        """
        ...


    def set(self, index: int, buffer: "ByteBuffer") -> "Vector2d":
        """
        Read this vector from the supplied ByteBuffer starting at the specified
        absolute buffer position/index.
        
        This method will not increment the position of the given ByteBuffer.

        Arguments
        - index: the absolute position into the ByteBuffer
        - buffer: values will be read in `x, y` order

        Returns
        - this
        """
        ...


    def set(self, buffer: "DoubleBuffer") -> "Vector2d":
        """
        Read this vector from the supplied DoubleBuffer at the current
        buffer DoubleBuffer.position() position.
        
        This method will not increment the position of the given DoubleBuffer.
        
        In order to specify the offset into the DoubleBuffer at which
        the vector is read, use .set(int, DoubleBuffer), taking
        the absolute position as parameter.

        Arguments
        - buffer: values will be read in `x, y` order

        Returns
        - this

        See
        - .set(int, DoubleBuffer)
        """
        ...


    def set(self, index: int, buffer: "DoubleBuffer") -> "Vector2d":
        """
        Read this vector from the supplied DoubleBuffer starting at the specified
        absolute buffer position/index.
        
        This method will not increment the position of the given DoubleBuffer.

        Arguments
        - index: the absolute position into the DoubleBuffer
        - buffer: values will be read in `x, y` order

        Returns
        - this
        """
        ...


    def setFromAddress(self, address: int) -> "Vector2d":
        """
        Set the values of this vector by reading 2 double values from off-heap memory,
        starting at the given address.
        
        This method will throw an UnsupportedOperationException when JOML is used with `-Djoml.nounsafe`.
        
        *This method is unsafe as it can result in a crash of the JVM process when the specified address range does not belong to this process.*

        Arguments
        - address: the off-heap memory address to read the vector values from

        Returns
        - this
        """
        ...


    def get(self, component: int) -> float:
        ...


    def get(self, mode: int, dest: "Vector2i") -> "Vector2i":
        ...


    def get(self, dest: "Vector2f") -> "Vector2f":
        ...


    def get(self, dest: "Vector2d") -> "Vector2d":
        ...


    def setComponent(self, component: int, value: float) -> "Vector2d":
        """
        Set the value of the specified component of this vector.

        Arguments
        - component: the component whose value to set, within `[0..1]`
        - value: the value to set

        Returns
        - this

        Raises
        - IllegalArgumentException: if `component` is not within `[0..1]`
        """
        ...


    def get(self, buffer: "ByteBuffer") -> "ByteBuffer":
        ...


    def get(self, index: int, buffer: "ByteBuffer") -> "ByteBuffer":
        ...


    def get(self, buffer: "DoubleBuffer") -> "DoubleBuffer":
        ...


    def get(self, index: int, buffer: "DoubleBuffer") -> "DoubleBuffer":
        ...


    def getToAddress(self, address: int) -> "Vector2dc":
        ...


    def perpendicular(self) -> "Vector2d":
        """
        Set this vector to be one of its perpendicular vectors.

        Returns
        - this
        """
        ...


    def sub(self, v: "Vector2dc") -> "Vector2d":
        """
        Subtract `v` from this vector.

        Arguments
        - v: the vector to subtract

        Returns
        - this
        """
        ...


    def sub(self, x: float, y: float) -> "Vector2d":
        """
        Subtract `(x, y)` from this vector.

        Arguments
        - x: the x component to subtract
        - y: the y component to subtract

        Returns
        - this
        """
        ...


    def sub(self, x: float, y: float, dest: "Vector2d") -> "Vector2d":
        ...


    def sub(self, v: "Vector2fc") -> "Vector2d":
        """
        Subtract `v` from this vector.

        Arguments
        - v: the vector to subtract

        Returns
        - this
        """
        ...


    def sub(self, v: "Vector2dc", dest: "Vector2d") -> "Vector2d":
        ...


    def sub(self, v: "Vector2fc", dest: "Vector2d") -> "Vector2d":
        ...


    def mul(self, scalar: float) -> "Vector2d":
        """
        Multiply the components of this vector by the given scalar.

        Arguments
        - scalar: the value to multiply this vector's components by

        Returns
        - this
        """
        ...


    def mul(self, scalar: float, dest: "Vector2d") -> "Vector2d":
        ...


    def mul(self, x: float, y: float) -> "Vector2d":
        """
        Multiply the components of this vector by the given scalar values and store the result in `this`.

        Arguments
        - x: the x component to multiply this vector by
        - y: the y component to multiply this vector by

        Returns
        - this
        """
        ...


    def mul(self, x: float, y: float, dest: "Vector2d") -> "Vector2d":
        ...


    def mul(self, v: "Vector2dc") -> "Vector2d":
        """
        Multiply this vector component-wise by another vector.

        Arguments
        - v: the vector to multiply by

        Returns
        - this
        """
        ...


    def mul(self, v: "Vector2dc", dest: "Vector2d") -> "Vector2d":
        ...


    def div(self, scalar: float) -> "Vector2d":
        """
        Divide this vector by the given scalar value.

        Arguments
        - scalar: the scalar to divide this vector by

        Returns
        - this
        """
        ...


    def div(self, scalar: float, dest: "Vector2d") -> "Vector2d":
        ...


    def div(self, x: float, y: float) -> "Vector2d":
        """
        Divide the components of this vector by the given scalar values and store the result in `this`.

        Arguments
        - x: the x component to divide this vector by
        - y: the y component to divide this vector by

        Returns
        - this
        """
        ...


    def div(self, x: float, y: float, dest: "Vector2d") -> "Vector2d":
        ...


    def div(self, v: "Vector2dc") -> "Vector2d":
        """
        Divide this vector component-wise by another vector.

        Arguments
        - v: the vector to divide by

        Returns
        - this
        """
        ...


    def div(self, v: "Vector2fc") -> "Vector2d":
        """
        Divide this vector component-wise by another Vector2fc.

        Arguments
        - v: the vector to divide by

        Returns
        - this
        """
        ...


    def div(self, v: "Vector2fc", dest: "Vector2d") -> "Vector2d":
        ...


    def div(self, v: "Vector2dc", dest: "Vector2d") -> "Vector2d":
        ...


    def mul(self, mat: "Matrix2fc") -> "Vector2d":
        """
        Multiply the given matrix `mat` with this vector.

        Arguments
        - mat: the matrix to multiply this vector by

        Returns
        - this
        """
        ...


    def mul(self, mat: "Matrix2dc") -> "Vector2d":
        """
        Multiply the given matrix `mat` with this vector.

        Arguments
        - mat: the matrix to multiply this vector by

        Returns
        - this
        """
        ...


    def mul(self, mat: "Matrix2dc", dest: "Vector2d") -> "Vector2d":
        ...


    def mul(self, mat: "Matrix2fc", dest: "Vector2d") -> "Vector2d":
        ...


    def mulTranspose(self, mat: "Matrix2dc") -> "Vector2d":
        """
        Multiply the transpose of the given matrix with this vector and store the result in `this`.

        Arguments
        - mat: the matrix

        Returns
        - this
        """
        ...


    def mulTranspose(self, mat: "Matrix2dc", dest: "Vector2d") -> "Vector2d":
        ...


    def mulTranspose(self, mat: "Matrix2fc") -> "Vector2d":
        """
        Multiply the transpose of the given matrix with  this vector and store the result in `this`.

        Arguments
        - mat: the matrix

        Returns
        - this
        """
        ...


    def mulTranspose(self, mat: "Matrix2fc", dest: "Vector2d") -> "Vector2d":
        ...


    def mulPosition(self, mat: "Matrix3x2dc") -> "Vector2d":
        """
        Multiply the given 3x2 matrix `mat` with `this`.
        
        This method assumes the `z` component of `this` to be `1.0`.

        Arguments
        - mat: the matrix to multiply this vector by

        Returns
        - this
        """
        ...


    def mulPosition(self, mat: "Matrix3x2dc", dest: "Vector2d") -> "Vector2d":
        ...


    def mulDirection(self, mat: "Matrix3x2dc") -> "Vector2d":
        """
        Multiply the given 3x2 matrix `mat` with `this`.
        
        This method assumes the `z` component of `this` to be `0.0`.

        Arguments
        - mat: the matrix to multiply this vector by

        Returns
        - this
        """
        ...


    def mulDirection(self, mat: "Matrix3x2dc", dest: "Vector2d") -> "Vector2d":
        ...


    def dot(self, v: "Vector2dc") -> float:
        ...


    def angle(self, v: "Vector2dc") -> float:
        ...


    def lengthSquared(self) -> float:
        ...


    @staticmethod
    def lengthSquared(x: float, y: float) -> float:
        """
        Get the length squared of a 2-dimensional double-precision vector.

    Author(s)
        - F. Neurath

        Arguments
        - x: The vector's x component
        - y: The vector's y component

        Returns
        - the length squared of the given vector
        """
        ...


    def length(self) -> float:
        ...


    @staticmethod
    def length(x: float, y: float) -> float:
        """
        Get the length of a 2-dimensional double-precision vector.

    Author(s)
        - F. Neurath

        Arguments
        - x: The vector's x component
        - y: The vector's y component

        Returns
        - the length of the given vector
        """
        ...


    def distance(self, v: "Vector2dc") -> float:
        ...


    def distanceSquared(self, v: "Vector2dc") -> float:
        ...


    def distance(self, v: "Vector2fc") -> float:
        ...


    def distanceSquared(self, v: "Vector2fc") -> float:
        ...


    def distance(self, x: float, y: float) -> float:
        ...


    def distanceSquared(self, x: float, y: float) -> float:
        ...


    @staticmethod
    def distance(x1: float, y1: float, x2: float, y2: float) -> float:
        """
        Return the distance between `(x1, y1)` and `(x2, y2)`.

        Arguments
        - x1: the x component of the first vector
        - y1: the y component of the first vector
        - x2: the x component of the second vector
        - y2: the y component of the second vector

        Returns
        - the euclidean distance
        """
        ...


    @staticmethod
    def distanceSquared(x1: float, y1: float, x2: float, y2: float) -> float:
        """
        Return the squared distance between `(x1, y1)` and `(x2, y2)`.

        Arguments
        - x1: the x component of the first vector
        - y1: the y component of the first vector
        - x2: the x component of the second vector
        - y2: the y component of the second vector

        Returns
        - the euclidean distance squared
        """
        ...


    def normalize(self) -> "Vector2d":
        """
        Normalize this vector.

        Returns
        - this
        """
        ...


    def normalize(self, dest: "Vector2d") -> "Vector2d":
        ...


    def normalize(self, length: float) -> "Vector2d":
        """
        Scale this vector to have the given length.

        Arguments
        - length: the desired length

        Returns
        - this
        """
        ...


    def normalize(self, length: float, dest: "Vector2d") -> "Vector2d":
        ...


    def add(self, v: "Vector2dc") -> "Vector2d":
        """
        Add `v` to this vector.

        Arguments
        - v: the vector to add

        Returns
        - this
        """
        ...


    def add(self, x: float, y: float) -> "Vector2d":
        """
        Add `(x, y)` to this vector.

        Arguments
        - x: the x component to add
        - y: the y component to add

        Returns
        - this
        """
        ...


    def add(self, x: float, y: float, dest: "Vector2d") -> "Vector2d":
        ...


    def add(self, v: "Vector2fc") -> "Vector2d":
        """
        Add `v` to this vector.

        Arguments
        - v: the vector to add

        Returns
        - this
        """
        ...


    def add(self, v: "Vector2dc", dest: "Vector2d") -> "Vector2d":
        ...


    def add(self, v: "Vector2fc", dest: "Vector2d") -> "Vector2d":
        ...


    def zero(self) -> "Vector2d":
        """
        Set all components to zero.

        Returns
        - this
        """
        ...


    def writeExternal(self, out: "ObjectOutput") -> None:
        ...


    def readExternal(self, in: "ObjectInput") -> None:
        ...


    def negate(self) -> "Vector2d":
        """
        Negate this vector.

        Returns
        - this
        """
        ...


    def negate(self, dest: "Vector2d") -> "Vector2d":
        ...


    def lerp(self, other: "Vector2dc", t: float) -> "Vector2d":
        """
        Linearly interpolate `this` and `other` using the given interpolation factor `t`
        and store the result in `this`.
        
        If `t` is `0.0` then the result is `this`. If the interpolation factor is `1.0`
        then the result is `other`.

        Arguments
        - other: the other vector
        - t: the interpolation factor between 0.0 and 1.0

        Returns
        - this
        """
        ...


    def lerp(self, other: "Vector2dc", t: float, dest: "Vector2d") -> "Vector2d":
        ...


    def hashCode(self) -> int:
        ...


    def equals(self, obj: "Object") -> bool:
        ...


    def equals(self, v: "Vector2dc", delta: float) -> bool:
        ...


    def equals(self, x: float, y: float) -> bool:
        ...


    def toString(self) -> str:
        """
        Return a string representation of this vector.
        
        This method creates a new DecimalFormat on every invocation with the format string "`0.000E0;-`".

        Returns
        - the string representation
        """
        ...


    def toString(self, formatter: "NumberFormat") -> str:
        """
        Return a string representation of this vector by formatting the vector components with the given NumberFormat.

        Arguments
        - formatter: the NumberFormat used to format the vector components with

        Returns
        - the string representation
        """
        ...


    def fma(self, a: "Vector2dc", b: "Vector2dc") -> "Vector2d":
        """
        Add the component-wise multiplication of `a * b` to this vector.

        Arguments
        - a: the first multiplicand
        - b: the second multiplicand

        Returns
        - this
        """
        ...


    def fma(self, a: float, b: "Vector2dc") -> "Vector2d":
        """
        Add the component-wise multiplication of `a * b` to this vector.

        Arguments
        - a: the first multiplicand
        - b: the second multiplicand

        Returns
        - this
        """
        ...


    def fma(self, a: "Vector2dc", b: "Vector2dc", dest: "Vector2d") -> "Vector2d":
        ...


    def fma(self, a: float, b: "Vector2dc", dest: "Vector2d") -> "Vector2d":
        ...


    def min(self, v: "Vector2dc") -> "Vector2d":
        """
        Set the components of this vector to be the component-wise minimum of this and the other vector.

        Arguments
        - v: the other vector

        Returns
        - this
        """
        ...


    def min(self, v: "Vector2dc", dest: "Vector2d") -> "Vector2d":
        ...


    def max(self, v: "Vector2dc") -> "Vector2d":
        """
        Set the components of this vector to be the component-wise maximum of this and the other vector.

        Arguments
        - v: the other vector

        Returns
        - this
        """
        ...


    def max(self, v: "Vector2dc", dest: "Vector2d") -> "Vector2d":
        ...


    def maxComponent(self) -> int:
        ...


    def minComponent(self) -> int:
        ...


    def floor(self) -> "Vector2d":
        """
        Set each component of this vector to the largest (closest to positive
        infinity) `double` value that is less than or equal to that
        component and is equal to a mathematical integer.

        Returns
        - this
        """
        ...


    def floor(self, dest: "Vector2d") -> "Vector2d":
        ...


    def ceil(self) -> "Vector2d":
        """
        Set each component of this vector to the smallest (closest to negative
        infinity) `double` value that is greater than or equal to that
        component and is equal to a mathematical integer.

        Returns
        - this
        """
        ...


    def ceil(self, dest: "Vector2d") -> "Vector2d":
        ...


    def round(self) -> "Vector2d":
        """
        Set each component of this vector to the closest double that is equal to
        a mathematical integer, with ties rounding to positive infinity.

        Returns
        - this
        """
        ...


    def round(self, dest: "Vector2d") -> "Vector2d":
        ...


    def isFinite(self) -> bool:
        ...


    def absolute(self) -> "Vector2d":
        """
        Set `this` vector's components to their respective absolute values.

        Returns
        - this
        """
        ...


    def absolute(self, dest: "Vector2d") -> "Vector2d":
        ...


    def clone(self) -> "Object":
        ...



"""
Python module generated from Java source file org.bukkit.plugin.java.JavaPlugin

Java source file obtained from artifact spigot-api version 1.21.4-R0.1-20250303.102353-42

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from com.google.common.base import Charsets
from com.google.common.base import Preconditions
from java.io import File
from java.io import FileOutputStream
from java.io import IOException
from java.io import InputStream
from java.io import InputStreamReader
from java.io import OutputStream
from java.io import Reader
from java.net import URL
from java.net import URLConnection
from java.util import Locale
from org.bukkit import Server
from org.bukkit.command import Command
from org.bukkit.command import CommandSender
from org.bukkit.command import PluginCommand
from org.bukkit.configuration.file import FileConfiguration
from org.bukkit.configuration.file import YamlConfiguration
from org.bukkit.generator import BiomeProvider
from org.bukkit.generator import ChunkGenerator
from org.bukkit.plugin import PluginBase
from org.bukkit.plugin import PluginDescriptionFile
from org.bukkit.plugin import PluginLoader
from org.bukkit.plugin import PluginLogger
from org.bukkit.plugin.java import *
from typing import Any, Callable, Iterable, Tuple


class JavaPlugin(PluginBase):
    """
    Represents a Java plugin and its main class. It contains fundamental methods
    and fields for a plugin to be loaded and work properly. This is an indirect
    implementation of org.bukkit.plugin.Plugin.
    """

    def __init__(self):
        ...


    def getDataFolder(self) -> "File":
        """
        Returns the folder that the plugin data's files are located in. The
        folder may not yet exist.

        Returns
        - The folder.
        """
        ...


    def getPluginLoader(self) -> "PluginLoader":
        """
        Gets the associated PluginLoader responsible for this plugin

        Returns
        - PluginLoader that controls this plugin
        """
        ...


    def getServer(self) -> "Server":
        """
        Returns the Server instance currently running this plugin

        Returns
        - Server running this plugin
        """
        ...


    def isEnabled(self) -> bool:
        """
        Returns a value indicating whether or not this plugin is currently
        enabled

        Returns
        - True if this plugin is enabled, otherwise False
        """
        ...


    def getDescription(self) -> "PluginDescriptionFile":
        """
        Returns the plugin.yaml file containing the details for this plugin

        Returns
        - Contents of the plugin.yaml file
        """
        ...


    def getConfig(self) -> "FileConfiguration":
        ...


    def reloadConfig(self) -> None:
        ...


    def saveConfig(self) -> None:
        ...


    def saveDefaultConfig(self) -> None:
        ...


    def saveResource(self, resourcePath: str, replace: bool) -> None:
        ...


    def getResource(self, filename: str) -> "InputStream":
        ...


    def onCommand(self, sender: "CommandSender", command: "Command", label: str, args: list[str]) -> bool:
        """

        """
        ...


    def onTabComplete(self, sender: "CommandSender", command: "Command", alias: str, args: list[str]) -> list[str]:
        """

        """
        ...


    def getCommand(self, name: str) -> "PluginCommand":
        """
        Gets the command with the given name, specific to this plugin. Commands
        need to be registered in the PluginDescriptionFile.getCommands()
        PluginDescriptionFile to exist at runtime.

        Arguments
        - name: name or alias of the command

        Returns
        - the plugin command if found, otherwise null
        """
        ...


    def onLoad(self) -> None:
        ...


    def onDisable(self) -> None:
        ...


    def onEnable(self) -> None:
        ...


    def getDefaultWorldGenerator(self, worldName: str, id: str) -> "ChunkGenerator":
        ...


    def getDefaultBiomeProvider(self, worldName: str, id: str) -> "BiomeProvider":
        ...


    def isNaggable(self) -> bool:
        ...


    def setNaggable(self, canNag: bool) -> None:
        ...


    def getLogger(self) -> "Logger":
        ...


    def toString(self) -> str:
        ...


    @staticmethod
    def getPlugin(clazz: type["T"]) -> "T":
        """
        This method provides fast access to the plugin that has .getProvidingPlugin(Class) provided the given plugin class, which is
        usually the plugin that implemented it.
        
        An exception to this would be if plugin's jar that contained the class
        does not extend the class, where the intended plugin would have
        resided in a different jar / classloader.
        
        Type `<T>`: a class that extends JavaPlugin

        Arguments
        - clazz: the class desired

        Returns
        - the plugin that provides and implements said class

        Raises
        - IllegalArgumentException: if clazz is null
        - IllegalArgumentException: if clazz does not extend JavaPlugin
        - IllegalStateException: if clazz was not provided by a plugin,
            for example, if called with
            `JavaPlugin.getPlugin(JavaPlugin.class)`
        - IllegalStateException: if called from the static initializer for
            given JavaPlugin
        - ClassCastException: if plugin that provided the class does not
            extend the class
        """
        ...


    @staticmethod
    def getProvidingPlugin(clazz: type[Any]) -> "JavaPlugin":
        """
        This method provides fast access to the plugin that has provided the
        given class.

        Arguments
        - clazz: a class belonging to a plugin

        Returns
        - the plugin that provided the class

        Raises
        - IllegalArgumentException: if the class is not provided by a
            JavaPlugin
        - IllegalArgumentException: if class is null
        - IllegalStateException: if called from the static initializer for
            given JavaPlugin
        """
        ...



"""
Python module generated from Java source file org.bukkit.block.Lectern

Java source file obtained from artifact spigot-api version 1.21.4-R0.1-20250303.102353-42

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from org.bukkit.block import *
from org.bukkit.inventory import BlockInventoryHolder
from org.bukkit.inventory import Inventory
from typing import Any, Callable, Iterable, Tuple


class Lectern(TileState, BlockInventoryHolder):
    """
    Represents a captured state of a lectern.
    """

    def getPage(self) -> int:
        """
        Get the current lectern page.

        Returns
        - current page
        """
        ...


    def setPage(self, page: int) -> None:
        """
        Set the current lectern page.
        
        If the page is greater than the number of pages of the book currently in
        the inventory, then behavior is undefined.

        Arguments
        - page: new page
        """
        ...


    def getInventory(self) -> "Inventory":
        """
        Returns
        - inventory

        See
        - Container.getInventory()
        """
        ...


    def getSnapshotInventory(self) -> "Inventory":
        """
        Returns
        - snapshot inventory

        See
        - Container.getSnapshotInventory()
        """
        ...



"""
Python module generated from Java source file com.google.common.base.Functions

Java source file obtained from artifact guava version 33.3.1-jre

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from com.google.common.annotations import GwtCompatible
from com.google.common.base import *
from java.io import Serializable
from javax.annotation import CheckForNull
from org.checkerframework.checker.nullness.qual import Nullable
from typing import Any, Callable, Iterable, Tuple


class Functions:
    """
    Static utility methods pertaining to `com.google.common.base.Function` instances; see that
    class for information about migrating to `java.util.function`.
    
    All methods return serializable functions as long as they're given serializable parameters.
    
    See the Guava User Guide article on <a
    href="https://github.com/google/guava/wiki/FunctionalExplained">the use of `Function`</a>.

    Author(s)
    - Jared Levy

    Since
    - 2.0
    """

    @staticmethod
    def toStringFunction() -> "Function"["Object", str]:
        """
        A function equivalent to the method reference `Object::toString`, for users not yet using
        Java 8. The function simply invokes `toString` on its argument and returns the result. It
        throws a NullPointerException on null input.
        
        **Warning:** The returned function may not be *consistent with equals* (as
        documented at Function.apply). For example, this function yields different results for
        the two equal instances `ImmutableSet.of(1, 2)` and `ImmutableSet.of(2, 1)`.
        
        **Warning:** as with all function types in this package, avoid depending on the specific
        `equals`, `hashCode` or `toString` behavior of the returned function. A
        future migration to `java.util.function` will not preserve this behavior.
        
        **Java 8+ users:** use the method reference `Object::toString` instead. In the
        future, when this class requires Java 8, this method will be deprecated. See Function
        for more important information about the Java 8+ transition.
        """
        ...


    @staticmethod
    def identity() -> "Function"["E", "E"]:
        ...


    @staticmethod
    def forMap(map: dict["K", "V"]) -> "Function"["K", "V"]:
        """
        Returns a function which performs a map lookup. The returned function throws an IllegalArgumentException if given a key that does not exist in the map. See also .forMap(Map, Object), which returns a default value in this case.
        
        Note: if `map` is a com.google.common.collect.BiMap BiMap (or can be one), you
        can use com.google.common.collect.Maps.asConverter Maps.asConverter instead to get a
        function that also supports reverse conversion.
        
        **Java 8+ users:** if you are okay with `null` being returned for an unrecognized
        key (instead of an exception being thrown), you can use the method reference `map::get`
        instead.
        """
        ...


    @staticmethod
    def forMap(map: dict["K", "V"], defaultValue: "V") -> "Function"["K", "V"]:
        """
        Returns a function which performs a map lookup with a default value. The function created by
        this method returns `defaultValue` for all inputs that do not belong to the map's key
        set. See also .forMap(Map), which throws an exception in this case.
        
        **Java 8+ users:** you can just write the lambda expression `k ->
        map.getOrDefault(k, defaultValue)` instead.

        Arguments
        - map: source map that determines the function behavior
        - defaultValue: the value to return for inputs that aren't map keys

        Returns
        - function that returns `map.get(a)` when `a` is a key, or `defaultValue` otherwise
        """
        ...


    @staticmethod
    def compose(g: "Function"["B", "C"], f: "Function"["A", "B"]) -> "Function"["A", "C"]:
        """
        Returns the composition of two functions. For `f: A->B` and `g: B->C`, composition
        is defined as the function h such that `h(a) == g(f(a))` for each `a`.
        
        **Java 8+ users:** use `g.compose(f)` or (probably clearer) `f.andThen(g)`
        instead.

        Arguments
        - g: the second function to apply
        - f: the first function to apply

        Returns
        - the composition of `f` and `g`

        See
        - <a href="//en.wikipedia.org/wiki/Function_composition">function composition</a>
        """
        ...


    @staticmethod
    def forPredicate(predicate: "Predicate"["T"]) -> "Function"["T", "Boolean"]:
        """
        Creates a function that returns the same boolean output as the given predicate for all inputs.
        
        The returned function is *consistent with equals* (as documented at Function.apply) if and only if `predicate` is itself consistent with equals.
        
        **Java 8+ users:** use the method reference `predicate::test` instead.
        """
        ...


    @staticmethod
    def constant(value: "E") -> "Function"["Object", "E"]:
        """
        Returns a function that ignores its input and always returns `value`.
        
        **Java 8+ users:** use the lambda expression `o -> value` instead.

        Arguments
        - value: the constant value for the function to return

        Returns
        - a function that always returns `value`
        """
        ...


    @staticmethod
    def forSupplier(supplier: "Supplier"["T"]) -> "Function"["F", "T"]:
        """
        Returns a function that ignores its input and returns the result of `supplier.get()`.
        
        **Java 8+ users:** use the lambda expression `o -> supplier.get()` instead.

        Since
        - 10.0
        """
        ...



"""
Python module generated from Java source file org.bukkit.material.types.MushroomBlockTexture

Java source file obtained from artifact spigot-api version 1.21.4-R0.1-20250303.102353-42

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from com.google.common.collect import Maps
from enum import Enum
from org.bukkit.block import BlockFace
from org.bukkit.material.types import *
from typing import Any, Callable, Iterable, Tuple


class MushroomBlockTexture(Enum):
    """
    Represents the different textured blocks of mushroom.
    """

    ALL_PORES = (0, None)
    """
    Pores on all faces.
    """
    CAP_NORTH_WEST = (1, BlockFace.NORTH_WEST)
    """
    Cap texture on the top, north and west faces, pores on remaining sides.
    """
    CAP_NORTH = (2, BlockFace.NORTH)
    """
    Cap texture on the top and north faces, pores on remaining sides.
    """
    CAP_NORTH_EAST = (3, BlockFace.NORTH_EAST)
    """
    Cap texture on the top, north and east faces, pores on remaining sides.
    """
    CAP_WEST = (4, BlockFace.WEST)
    """
    Cap texture on the top and west faces, pores on remaining sides.
    """
    CAP_TOP = (5, BlockFace.UP)
    """
    Cap texture on the top face, pores on remaining sides.
    """
    CAP_EAST = (6, BlockFace.EAST)
    """
    Cap texture on the top and east faces, pores on remaining sides.
    """
    CAP_SOUTH_WEST = (7, BlockFace.SOUTH_WEST)
    """
    Cap texture on the top, south and west faces, pores on remaining sides.
    """
    CAP_SOUTH = (8, BlockFace.SOUTH)
    """
    Cap texture on the top and south faces, pores on remaining sides.
    """
    CAP_SOUTH_EAST = (9, BlockFace.SOUTH_EAST)
    """
    Cap texture on the top, south and east faces, pores on remaining sides.
    """
    STEM_SIDES = (10, None)
    """
    Stem texture on the north, east, south and west faces, pores on top and
    bottom.
    """
    ALL_CAP = (14, BlockFace.SELF)
    """
    Cap texture on all faces.
    """
    ALL_STEM = (15, None)
    """
    Stem texture on all faces.
    """


    def getData(self) -> int:
        """
        Gets the associated data value representing this mushroom block face.

        Returns
        - A byte containing the data value of this mushroom block face

        Deprecated
        - Magic value
        """
        ...


    def getCapFace(self) -> "BlockFace":
        """
        Gets the face that has cap texture.

        Returns
        - The cap face
        """
        ...


    @staticmethod
    def getByData(data: int) -> "MushroomBlockTexture":
        """
        Gets the MushroomBlockType with the given data value.

        Arguments
        - data: Data value to fetch

        Returns
        - The MushroomBlockTexture representing the given value, or
        null if it doesn't exist

        Deprecated
        - Magic value
        """
        ...


    @staticmethod
    def getCapByFace(face: "BlockFace") -> "MushroomBlockTexture":
        """
        Gets the MushroomBlockType with cap texture on the given block face.

        Arguments
        - face: the required block face with cap texture

        Returns
        - The MushroomBlockTexture representing the given block
        face, or null if it doesn't exist

        See
        - BlockFace
        """
        ...



"""
Python module generated from Java source file org.bukkit.event.inventory.InventoryMoveItemEvent

Java source file obtained from artifact spigot-api version 1.21.4-R0.1-20250303.102353-42

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from com.google.common.base import Preconditions
from org.bukkit.event import Cancellable
from org.bukkit.event import Event
from org.bukkit.event import HandlerList
from org.bukkit.event.inventory import *
from org.bukkit.inventory import Inventory
from org.bukkit.inventory import ItemStack
from typing import Any, Callable, Iterable, Tuple


class InventoryMoveItemEvent(Event, Cancellable):
    """
    Called when some entity or block (e.g. hopper) tries to move items directly
    from one inventory to another.
    
    When this event is called, the initiator may already have removed the item
    from the source inventory and is ready to move it into the destination
    inventory.
    
    If this event is cancelled, the items will be returned to the source
    inventory, if needed.
    
    If this event is not cancelled, the initiator will try to put the ItemStack
    into the destination inventory. If this is not possible and the ItemStack
    has not been modified, the source inventory slot will be restored to its
    former state. Otherwise any additional items will be discarded.
    """

    def __init__(self, sourceInventory: "Inventory", itemStack: "ItemStack", destinationInventory: "Inventory", didSourceInitiate: bool):
        ...


    def getSource(self) -> "Inventory":
        """
        Gets the Inventory that the ItemStack is being taken from

        Returns
        - Inventory that the ItemStack is being taken from
        """
        ...


    def getItem(self) -> "ItemStack":
        """
        Gets the ItemStack being moved; if modified, the original item will not
        be removed from the source inventory.

        Returns
        - ItemStack
        """
        ...


    def setItem(self, itemStack: "ItemStack") -> None:
        """
        Sets the ItemStack being moved; if this is different from the original
        ItemStack, the original item will not be removed from the source
        inventory.

        Arguments
        - itemStack: The ItemStack
        """
        ...


    def getDestination(self) -> "Inventory":
        """
        Gets the Inventory that the ItemStack is being put into

        Returns
        - Inventory that the ItemStack is being put into
        """
        ...


    def getInitiator(self) -> "Inventory":
        """
        Gets the Inventory that initiated the transfer. This will always be
        either the destination or source Inventory.

        Returns
        - Inventory that initiated the transfer
        """
        ...


    def isCancelled(self) -> bool:
        ...


    def setCancelled(self, cancel: bool) -> None:
        ...


    def getHandlers(self) -> "HandlerList":
        ...


    @staticmethod
    def getHandlerList() -> "HandlerList":
        ...



"""
Python module generated from Java source file org.bukkit.entity.Tadpole

Java source file obtained from artifact spigot-api version 1.21.4-R0.1-20250303.102353-42

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from org.bukkit.entity import *
from typing import Any, Callable, Iterable, Tuple


class Tadpole(Fish):
    """
    A baby Frog.
    """

    def getAge(self) -> int:
        """
        Gets the age of this mob.

        Returns
        - Age
        """
        ...


    def setAge(self, age: int) -> None:
        """
        Sets the age of this mob.

        Arguments
        - age: New age
        """
        ...



"""
Python module generated from Java source file com.google.common.util.concurrent.ThreadFactoryBuilder

Java source file obtained from artifact guava version 33.3.1-jre

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from com.google.common.annotations import GwtIncompatible
from com.google.common.annotations import J2ktIncompatible
from com.google.common.util.concurrent import *
from com.google.errorprone.annotations import CanIgnoreReturnValue
from java.util import Locale
from java.util.concurrent import Executors
from java.util.concurrent import ThreadFactory
from java.util.concurrent.atomic import AtomicLong
from javax.annotation import CheckForNull
from typing import Any, Callable, Iterable, Tuple


class ThreadFactoryBuilder:
    """
    A ThreadFactory builder, providing any combination of these features:
    
    
      - whether threads should be marked as Thread.setDaemon daemon threads
      - a ThreadFactoryBuilder.setNameFormat naming format
      - a Thread.setPriority thread priority
      - an Thread.setUncaughtExceptionHandler uncaught exception handler
      - a ThreadFactory.newThread backing thread factory
    
    
    If no backing thread factory is provided, a default backing thread factory is used as if by
    calling `setThreadFactory(`Executors.defaultThreadFactory()`)`.

    Author(s)
    - Kurt Alfred Kluever

    Since
    - 4.0
    """

    def __init__(self):
        """
        Creates a new ThreadFactory builder.
        """
        ...


    def setNameFormat(self, nameFormat: str) -> "ThreadFactoryBuilder":
        """
        Sets the naming format to use when naming threads (Thread.setName) which are created
        with this ThreadFactory.

        Arguments
        - nameFormat: a String.format(String, Object...)-compatible format String, to which
            a unique integer (0, 1, etc.) will be supplied as the single parameter. This integer will
            be unique to the built instance of the ThreadFactory and will be assigned sequentially. For
            example, `"rpc-pool-%d"` will generate thread names like `"rpc-pool-0"`, `"rpc-pool-1"`, `"rpc-pool-2"`, etc.

        Returns
        - this for the builder pattern
        """
        ...


    def setDaemon(self, daemon: bool) -> "ThreadFactoryBuilder":
        """
        Sets daemon or not for new threads created with this ThreadFactory.

        Arguments
        - daemon: whether or not new Threads created with this ThreadFactory will be daemon threads

        Returns
        - this for the builder pattern
        """
        ...


    def setPriority(self, priority: int) -> "ThreadFactoryBuilder":
        """
        Sets the priority for new threads created with this ThreadFactory.
        
        **Warning:** relying on the thread scheduler is <a
        href="http://errorprone.info/bugpattern/ThreadPriorityCheck">discouraged</a>.

        Arguments
        - priority: the priority for new Threads created with this ThreadFactory

        Returns
        - this for the builder pattern
        """
        ...


    def setUncaughtExceptionHandler(self, uncaughtExceptionHandler: "UncaughtExceptionHandler") -> "ThreadFactoryBuilder":
        """
        Sets the UncaughtExceptionHandler for new threads created with this ThreadFactory.

        Arguments
        - uncaughtExceptionHandler: the uncaught exception handler for new Threads created with
            this ThreadFactory

        Returns
        - this for the builder pattern
        """
        ...


    def setThreadFactory(self, backingThreadFactory: "ThreadFactory") -> "ThreadFactoryBuilder":
        """
        Sets the backing ThreadFactory for new threads created with this ThreadFactory. Threads
        will be created by invoking #newThread(Runnable) on this backing ThreadFactory.

        Arguments
        - backingThreadFactory: the backing ThreadFactory which will be delegated to during
            thread creation.

        Returns
        - this for the builder pattern

        See
        - MoreExecutors
        """
        ...


    def build(self) -> "ThreadFactory":
        """
        Returns a new thread factory using the options supplied during the building process. After
        building, it is still possible to change the options used to build the ThreadFactory and/or
        build again. State is not shared amongst built instances.

        Returns
        - the fully constructed ThreadFactory
        """
        ...



"""
Python module generated from Java source file org.bukkit.damage.DamageSource

Java source file obtained from artifact spigot-api version 1.21.4-R0.1-20250303.102353-42

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from org.bukkit import Bukkit
from org.bukkit import Location
from org.bukkit.damage import *
from org.bukkit.entity import Entity
from typing import Any, Callable, Iterable, Tuple


class DamageSource:
    """
    Represents a source of damage.
    """

    def getDamageType(self) -> "DamageType":
        """
        Get the DamageType.

        Returns
        - the damage type
        """
        ...


    def getCausingEntity(self) -> "Entity":
        """
        Get the Entity that caused the damage to occur.
        
        Not to be confused with .getDirectEntity(), the causing entity is
        the entity to which the damage is ultimately attributed if the receiver
        is killed. If, for example, the receiver was damaged by a projectile, the
        shooter/thrower would be returned.

        Returns
        - an Entity or null
        """
        ...


    def getDirectEntity(self) -> "Entity":
        """
        Get the Entity that directly caused the damage.
        
        Not to be confused with .getCausingEntity(), the direct entity is
        the entity that actually inflicted the damage. If, for example, the
        receiver was damaged by a projectile, the projectile would be returned.

        Returns
        - an Entity or null
        """
        ...


    def getDamageLocation(self) -> "Location":
        """
        Get the Location from where the damage originated. This will only
        be present if an entity did not cause the damage.

        Returns
        - the location, or null if none
        """
        ...


    def getSourceLocation(self) -> "Location":
        """
        Get the Location from where the damage originated.
        
        This is a convenience method to get the final location of the damage.
        This method will attempt to return
        .getDamageLocation() the damage location. If this is null, the
        .getCausingEntity() causing entity location will be returned.
        Finally if there is no damage location nor a causing entity, null will be
        returned.

        Returns
        - the source of the location or null.
        """
        ...


    def isIndirect(self) -> bool:
        """
        Get if this damage is indirect.
        
        Damage is considered indirect if .getCausingEntity() is not equal
        to .getDirectEntity(). This will be the case, for example, if a
        skeleton shot an arrow or a player threw a potion.

        Returns
        - `True` if is indirect, `False` otherwise.
        """
        ...


    def getFoodExhaustion(self) -> float:
        """
        Get the amount of hunger exhaustion caused by this damage.

        Returns
        - the amount of hunger exhaustion caused.
        """
        ...


    def scalesWithDifficulty(self) -> bool:
        """
        Gets if this source of damage scales with difficulty.

        Returns
        - `True` if scales.
        """
        ...


    @staticmethod
    def builder(damageType: "DamageType") -> "Builder":
        """
        Create a new DamageSource.Builder.

        Arguments
        - damageType: the DamageType to use

        Returns
        - a DamageSource.Builder
        """
        ...


    class Builder:
        """
        Utility class to make building a DamageSource easier. Only a
        DamageType is required.
        """

        def withCausingEntity(self, entity: "Entity") -> "Builder":
            """
            Set the Entity that caused the damage.

            Arguments
            - entity: the entity

            Returns
            - this instance. Allows for chained method calls

            See
            - DamageSource.getCausingEntity()
            """
            ...


        def withDirectEntity(self, entity: "Entity") -> "Builder":
            """
            Set the Entity that directly inflicted the damage.

            Arguments
            - entity: the entity

            Returns
            - this instance. Allows for chained method calls

            See
            - DamageSource.getDirectEntity()
            """
            ...


        def withDamageLocation(self, location: "Location") -> "Builder":
            """
            Set the Location of the source of damage.

            Arguments
            - location: the location where the damage occurred

            Returns
            - this instance. Allows for chained method calls

            See
            - DamageSource.getSourceLocation()
            """
            ...


        def build(self) -> "DamageSource":
            """
            Create a new DamageSource instance using the supplied
            parameters.

            Returns
            - the damage source instance
            """
            ...



"""
Python module generated from Java source file org.bukkit.event.entity.EntityToggleGlideEvent

Java source file obtained from artifact spigot-api version 1.21.4-R0.1-20250303.102353-42

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from org.bukkit.entity import LivingEntity
from org.bukkit.event import Cancellable
from org.bukkit.event import HandlerList
from org.bukkit.event.entity import *
from typing import Any, Callable, Iterable, Tuple


class EntityToggleGlideEvent(EntityEvent, Cancellable):
    """
    Sent when an entity's gliding status is toggled with an Elytra.
    Examples of when this event would be called:
    
        - Player presses the jump key while in midair and using an Elytra
        - Player lands on ground while they are gliding (with an Elytra)
    
    This can be visually estimated by the animation in which a player turns horizontal.
    """

    def __init__(self, who: "LivingEntity", isGliding: bool):
        ...


    def isCancelled(self) -> bool:
        ...


    def setCancelled(self, cancel: bool) -> None:
        ...


    def isGliding(self) -> bool:
        """
        Returns True if the entity is now gliding or
        False if the entity stops gliding.

        Returns
        - new gliding state
        """
        ...


    def getHandlers(self) -> "HandlerList":
        ...


    @staticmethod
    def getHandlerList() -> "HandlerList":
        ...



"""
Python module generated from Java source file org.bukkit.ServerLinks

Java source file obtained from artifact spigot-api version 1.21.4-R0.1-20250303.102353-42

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from enum import Enum
from java.net import URI
from org.bukkit import *
from typing import Any, Callable, Iterable, Tuple


class ServerLinks:
    """
    Represents a collections of links which may be sent to a client.
    """

    def getLink(self, type: "Type") -> "ServerLink":
        """
        Gets the link of a given type, if it exists.

        Arguments
        - type: link type

        Returns
        - link or null
        """
        ...


    def getLinks(self) -> list["ServerLink"]:
        """
        Gets an immutable list of all links.

        Returns
        - immutable list
        """
        ...


    def setLink(self, type: "Type", url: "URI") -> "ServerLink":
        """
        Adds the given link, overwriting the first link of the same type if
        already set.

        Arguments
        - type: link type
        - url: link url

        Returns
        - the added link
        """
        ...


    def addLink(self, type: "Type", url: "URI") -> "ServerLink":
        """
        Adds the given link to the list of links.

        Arguments
        - type: link type
        - url: link url

        Returns
        - the added link
        """
        ...


    def addLink(self, displayName: str, url: "URI") -> "ServerLink":
        """
        Adds the given link to the list of links.

        Arguments
        - displayName: link name / display text
        - url: link url

        Returns
        - the added link
        """
        ...


    def removeLink(self, link: "ServerLink") -> bool:
        """
        Removes the given link.

        Arguments
        - link: the link to remove

        Returns
        - if the link existed and was removed
        """
        ...


    def copy(self) -> "ServerLinks":
        """
        Returns a copy of this link collection, unassociated from the server.

        Returns
        - copied links
        """
        ...


    class ServerLink:
        """
        Represents a server link.
        """

        def getType(self) -> "Type":
            """
            Gets the type of this link if it is a known special type.

            Returns
            - type or null
            """
            ...


        def getDisplayName(self) -> str:
            """
            Gets the display name/text of this link.

            Returns
            - display name
            """
            ...


        def getUrl(self) -> "URI":
            """
            Gets the url of this link.

            Returns
            - link url
            """
            ...


    class Type(Enum):
        """
        Represents a known type of link which will be translated by the client
        and may have special functionality.
        """

        REPORT_BUG = 0
        """
        Bug report links which may appear on disconnect/crash screens.
        """
        COMMUNITY_GUIDELINES = 1
        SUPPORT = 2
        STATUS = 3
        FEEDBACK = 4
        COMMUNITY = 5
        WEBSITE = 6
        FORUMS = 7
        NEWS = 8
        ANNOUNCEMENTS = 9



"""
Python module generated from Java source file org.bukkit.permissions.ServerOperator

Java source file obtained from artifact spigot-api version 1.21.4-R0.1-20250303.102353-42

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from org.bukkit.entity import Player
from org.bukkit.permissions import *
from typing import Any, Callable, Iterable, Tuple


class ServerOperator:
    """
    Represents an object that may become a server operator, such as a Player
    """

    def isOp(self) -> bool:
        """
        Checks if this object is a server operator

        Returns
        - True if this is an operator, otherwise False
        """
        ...


    def setOp(self, value: bool) -> None:
        """
        Sets the operator status of this object

        Arguments
        - value: New operator value
        """
        ...
